#!/bin/env perl
#//////////////////////////////////////////////////////////////////////////////
#
#  SilverStream installation script.
#
#  Copyright (c) 1997,1998 SilverStream Software, Inc., All Rights Reserved
#

$usage = "Usage: $0 [installdir]\n";
$quiet = "yes";

$invokekey = "invokefromsh";

($installdir, $srcdir, $cwd) = &ParseCmdLine(@ARGV);

&GlobalDefines();

#$producttype = &GetProductType($srcdir);
&AskQuestions($licenseagreement);

&BackupFile("$installdir/$PROPS_PATH/$HPROPS_FILENAME");
&BackupFile("$installdir/$PROPS_PATH/$SPROPS_FILENAME");
&BackupFile("$installdir/$PROPS_PATH/$SMCPROPS_FILENAME");
&DeleteFiles($installdir, \@deloldfiles);
$err_untar = &UnPackFile($srcdir, $tarfile, $jretarfile);

if ( -f "$srcdir/$ftstarfile" || -f "$srcdir/$ftstarfile.Z"){
    $err_untar += &UnPackFile($srcdir, $ftstarfile);
}

if ($installdocs eq "yes") {
    &UnPackFile($srcdir, $doctarfile);
}

if ($installsamples eq "yes") {
    &Copydir($srcdir, $installdir, "Samples");
}

&CopyExportDomestic($srcdir, $installdir);
&CopyFilesToJre($srcdir, $installdir);
if ( -d "$srcdir/WSI" ){
    system(qq'$cp -r "$srcdir/WSI" "$installdir"');
}

&LinkDocs($srcdir, $installdir);

&CopyEEfiles($srcdir, $installdir);
if ($oracthin == 1){ # 1 = oci, not thin
    &CopyOracleLib($installdir, $oraclehome);
}
&EditHttpdPropFile($installdir);
&EditSminitPropFile($installdir);
&EditSmcPropsFile($installdir);

print " + CLASSPATH = $ENV{CLASSPATH}\n";
print " + $libpathev = $ENV{$libpathev}\n";

&WriteProfileScript($installdir);

&WriteScriptFiles($installdir,
		    $masterName, $dbUsername, $dbPassword, $dbtype,
		    $java_version);

&WriteSpecialFile($installdir);
$end_msg = &WriteStartupScripts($installdir, $os_type);

print " + $cp $srcdir/version.txt $installdir/version.txt\n";
system(qq'$cp "$srcdir/version.txt" "$installdir/version.txt"');
if (  !(-d "$installdir/tmp") ) {
   system(qq'$mkdir "$installdir/tmp"');
}
system(qq'$cp "$srcdir/version.txt" "$installdir/tmp/"');

if ( $runSmInit eq "yes"){
    $err_sminit = &RunSilverMasterInit($installdir, $dbUsername, $dbPassword);
}
else{
    &RunAgDbRegistry($installdir, $dbUsername, $dbPassword, $masterName);
}

unless ($err_sminit || $updateassurance){
    $err_lic = &RunAgUploadLicense($licensesh, $srcdir, $installdir,
                        $licenseStr, $masterName, $dbUsername, $dbPassword);
}

&InstalljbrokerMQ($srcdir, $installdir);

if ( $> == 0 ){ # effective user id == root
    print qq' + $chown -R 0 "$installdir"\n';
    system(qq'$chown -R 0 "$installdir"');
    print qq' + $chgrp -R 0 "$installdir"\n';
    system(qq'$chgrp -R 0 "$installdir"');
}

print "$end_msg\n";
print "-" x 70 . "\n";
print "SilverStream Server installation finished: $installdir\n";
if ($err_untar){
    print " An error was detected while Unpacking the files ($err_untar)\n";
}
elsif ($err_sminit){
    print " An error was detected while running SilverMasterInit".
	    " ($err_sminit)\n";
}
elsif ($err_lic){
    print " An error was detected while running $licensesh ($err_lic)\n";
}
else{

# the server is started from the install script ...  rwood 2007-12-07
#    &StartServer($installdir);

    print "
    Please register your product at:
        http://www.silverstream.com/productreg.html\n\n";
}
###########################################################################
# Global Defines
sub GlobalDefines{

    $uname = "/bin/uname";
    $showrev = "/usr/bin/showrev";  # Solaris only
    $shell = "/bin/ksh";  # changed for Linux
    $cksum = "/usr/bin/cksum";
    $chown = "/bin/chown";
    $chgrp = "/bin/chgrp";

    #$licensecmd = "$srcdir/AgUploadLicense";
    $licensecmd = "upldlic.ag";
    $licensesh = "upldlic.sh";
    $licensecksum = "upldlic.cksum";
    $licenseagreement = "$srcdir/Docs/licenseu.txt";

    #$ENV{JAVA_COMPILER} = "sunwjit";

    # this file in the installdir will over ride any defaults below
    $installdefaults = ".aginstdefs";

    $os_type = qx`$uname -s`;
    $hardware_type = qx`$uname -m`;
    chomp($os_type);

    if ( $os_type =~ /SUN/i ){
	$libpathev = "LD_LIBRARY_PATH";
	chomp($platformType = qx`uname -p`);
        $subPathForCommonJavaLibs =  "lib/$platformType";
	$subPathForHotSpotServerVm = "lib/$platformType/server";
	$subPathForHotSpotClientVm = "lib/$platformType/client";
    }
    elsif ( $os_type =~ /HP/i ){
	$libpathev = "SHLIB_PATH";
        $subPathForCommonJavaLibs  = "lib/\$HPproc";
	$subPathForHotSpotServerVm = "lib/\$HPproc/hotspot";
	$subPathForHotSpotClientVm = "lib/\$HPproc/hotspot";
	$subPathForClassicVm       = "lib/\$HPproc/classic";  # Classic VM is a special case for HP.
    }
    elsif ( $os_type =~ /AIX/i ){
	$libpathev = "LIBPATH";
	#$ENV{JAVA_COMPILER} = "jitc"; # ppr 30275
	#$ENV{JAVA_COMPILER} = "NONE";
        $subPathForCommonJavaLibs  = "bin";
	$subPathForHotSpotServerVm = "bin/classic";
	$subPathForHotSpotClientVm = "bin/classic";
    }
    elsif ( $os_type =~ /Linux/i ){
	$libpathev = "LD_LIBRARY_PATH";
	$shell = "/bin/bash";
        $subPathForCommonJavaLibs  = "lib/i386";
	$subPathForHotSpotServerVm = "lib/i386/server";
	$subPathForHotSpotClientVm = "lib/i386/client";
    }
    else{
        gracefulexit(11, " ** Error: unknown system type: uname -s returned: $os_type\n");
    }

    $relnotes = "$srcdir/Docs/relnotes.html";
    $installdocs_default = "no";
    $installsamples_default = "no";
#NJR    $configjbmq = "yes";
    $configjbmq = "no";

    $ENV{DISPLAY} = ":0.0" unless $ENV{DISPLAY};

    # http://java.sun.com/j2se/1.3/install-solaris-patches.html

    # Solaris 5.6 == Solaris 2.6
    @sunpatches56 = qw(
	105633-56
	106040-16
	105568-23
	106842-09
	106841-01
	106409-01
	107733-09
	105210-38
	105181-26
	105669-10
	105284-41
	108091-03
	105591-09
	106125-11
	106429-02
    );

    # Solaris 7 or 2.7
    @sunpatches57 = qw(
	107226-17
	106980-16
	106541-16 
	108374-05
	107153-01
	107636-07
	107544-03
	108376-24
	106950-13
	107081-33
	106300-09
	106327-08
	107656-07
	107702-07
    );

    # Solaris 8 or 2.8
    @sunpatches58 = qw(
	108652-33
	108921-12
	108940-24
    );

    @solintelpatches56 = qw(
	106248-42
	106041-15
	105569-23
	106844-05
	105200-12
	106410-01
	107734-09
	105211-39
	105182-25
	105670-09
	105285-41
	108642-03
	104678-07
	106126-11
    );

    @solintelpatches57 = qw(
	107227-17
	106981-17
	106542-16
	108375-05
	107152-01
	107637-07
	107545-03
	108377-23
	106951-13
	107082-33
	106328-08
	107657-06
	107703-07
    );

    @solintelpatches58 = qw(
	108653-28
	108922-12
	108941-24
    );

    $default_installdir = "/opt/silverstream";
    $installdir = $default_installdir unless $installdir;

    $product_name = "SilverStream Server";
    $tarfile = "ag_tar";
    $ftstarfile = "fts_tar";
    $jretarfile = "jre.tar";
    $doctarfile = "Docs.tar";
    $agprofile = ".agprofile";
    $registryfile = ".SilverStream.reg";

    $zcat = "/bin/zcat";
    $tar  = "/bin/tar";
    $taropt = "xvf";
    $mkdir = "/bin/mkdir";
    $cp = "/bin/cp";
    $more = "/bin/more";

    $PROPS_PATH		= "Resources";
    $HPROPS_FILENAME	= "httpd.props";
    $SPROPS_FILENAME	= "sminit.props";
    $SMCPROPS_FILENAME	= "smc.props";
    $SYBASE_JDBC_URL	= "jdbc:sybase:Tds:";
    $INFORMIX_JDBC_URL	= "jdbc:informix-sqli:";
    $IBM_DB2_URL	= "jdbc:db2:";
    $CLOUDSCAPE_URL	= "jdbc:cloudscape:";
    $ORACLE_JDBC_URL	= "jdbc:sssw:oracle:";
    $ORACLE_THIN_URL	= "jdbc:oracle:thin:";
    $ORACLE_OCI_URL	= "jdbc:oracle:oci:";
    $MS_SQL2k_URL	= "jdbc:microsoft:sqlserver:";

    $URL_ORACLE_PORT	= "1521";
    $URL_ORACLE_SID	= "orcl";

    $SYBASE_PORT	= "5000";
    $SYBASEASA_PORT	= "2638";
    $INFORMIX_PORT	= "1526";
    $MSSQL_PORT		= "1433";

    #$AGPROPS_FILENAME	= "AgSystemIni.props";
    $AGPROPS_TMP_STRING	= "TempDirectory";
    $AGPROPS_SVR_STRING	= "Server.InstallPath";

    $SUNRMI_PROP_FILE = "sunrmi.props";
    #$SUNORB_PROP_FILE = "sunorb.props";
    $JBROKER_PROP_FILE = "jbroker.props";

    $CACHE_ENABLE	= "true";
    $CACHE_MEMCONTSIZE	= "131072";
    $CACHE_MEMSIZE	= "4194304";
    $CACHE_DISKCONTSIZE	= "2097158";
    $CACHE_DISKSIZE	= "20000000";
    $CACHE_DISKDIR	= "ContentCache";

    $EXE_DBREGY		= "AgDBRegistry";
    $EXE_UPGRDSCHMA	= "SilverUpgradeSchema";
    #$EXE_UPGRDAPP	= "SilverUpgradeApp";
    #$EXE_UPGRDSCHMA  = "AgConvertDb";
    #$EXE_UPGRDAPP  = "AgMDUpgrade";
    $SCRIPTFILE_FIXDB	= "SilverFixDb.sh";
    $CLASSNAME_FIXDB	= "com.sssw.srv.dbtools.AgFixDB";
    $SCRIPTFILE_RENAMEDB = "SilverRenameDb.sh";
    $CLASSNAME_RENAMEDB	= "com.sssw.tools.db.AgChangeDBName";

    $SCRIPTFILE_SSERVER	= "SilverServer";
    $CLASSNAME_SSERVER	= "com.sssw.srv.http.httpd";

    $SCRIPTFILE_SMINIT	= "SilverMasterInit";
    $CLASSNAME_SMINIT	= "com.sssw.srv.dbboot.BootCreateWebbase";

    $AG_ORACLELIB = "libagora8_jni.so";
    $AG_ORACLELIB8 = "libagora8_jni.so.8";
    $AG_ORACLELIB8i = "libagora8_jni.so.8i";
    $ORACLELIB8 = "libclntsh.so.1.0";
    $ORACLELIB8i = "libclntsh.so.8.0";

    $LDSKEY_ORACLE7		= "OraV7AgDriver";
    $LDSKEY_ORACLE7_CUST	= "OraV7AgCustomDriver";
    $LDSKEY_ORACLE8		= "OraV8AgDriver";
    $LDSKEY_ORACLE8_SQI		= "OraV8AgDriverSynonymQuoteIdentifiers";
    $LDSKEY_ORACLE8_CUST	= "OraV8AgCustomDriver";
    $LDSKEY_ORACLE8_THIN	= "OraV8OracleThin";
    $LDSKEY_ORACLE8_TSQI	= "OraV8OracleThinSynonymQuoteIdentifiers";
    $LDSKEY_ORACLE9_THIN	= "OraV9OracleThin";
    $LDSKEY_ORACLE9_TSQI	= "OraV9OracleThinSynonymQuoteIdentifiers";
    $LDSKEY_ORACLE9_OCI		= "OraV9OracleOCI";
    #$LDSKEY_ORACLE9_OCI_SYN	= "OraV9OracleOCISynonymQuoteIdentifiers";

    $LDSKEY_SYB_ASA6_JCON	= "SybAdaptiveV6JConnect5";
    $LDSKEY_SYB_ASA7_JCON	= "SybAdaptiveV7JConnect5";
    $LDSKEY_SYB_ASA8_JCON	= "SybAdaptiveV8JConnect5";
    $LDSKEY_SYBASE_SYS_JCON4	= "SybSQLSvrV11JConnect4";
    $LDSKEY_SYBASE_SYS_JCON5	= "SybSQLSvrV11JConnect5";
    $LDSKEY_SYBASE_SYS12_JCON4	= "SybSQLSvrV12JConnect4";
    $LDSKEY_SYBASE_SYS12_JCON5	= "SybSQLSvrV12JConnect5";
    #$LDSKEY_INFORMIX	= "InformixV7AgBridge";
    $LDSKEY_INFORMIX	= "InformixV7Type4";
    $LDSKEY_IBM_DB2_5	= "IBMDB2V5";
    $LDSKEY_IBM_DB2_6	= "IBMDB2V6";
    $LDSKEY_CLOUDSCAPE	= "CloudscapeEmbeddedDriver";
    $LDSKEY_MS_SQL2k_MS	= "MSSQLSvrV2000";

    $ORAC_ZIP_A = "classes12";
    $ORAC_ZIP_B = "nls_charset12";
    $orac_zip_a_subpath = "jdbc/lib";
    $orac_zip_a_dir = "";
    $orac_zip_b_dir = "";

    $FILE = 2;
    $DIRECTORY = 3;
    $INTEGER = 4;
    $BLANK = 5;
    $YESNO = 6;
    $BACK = "gobacktotheprevioussubroutine"; # just something unique

    $dbtype = "Oracle8"; # or Sybase or Informix
    $oracthin = "1"; # 1=silverstream driver, 2=oracle thin driver
    $oracsyn = "no";
    $oracsrv = "";
    $oracport = "";
    $oracsid = "";
    $oractns = "";

    #
    # some default answers
    #
    $masterName = "SilverMaster";
    #$dbUsername = "sa";
    #$dbPassword = "password";
#NJR
    $dbPassword = "pv";

    if ( -f "$srcdir/license_string.txt"){
	open(LIC, "$srcdir/license_string.txt");
	chomp($licenseStr = <LIC>);
	close(LIC);
	$licenseStr =~ s/\W$//;
    }

    $informixjar = "ifxjdbc.jar";
    #$j4checkfile = "com/sybase/jdbc/SybDriver.class";
    $ibmcheckfile = "sqllib/java/db2java.zip";
    #$sybasedir = "/usr/sybase/classes";
    #$sybasedir = "/usr/local/sybase/classes";
    $sybjconn2jar = "jconn2.jar";
    if ( $os_type =~ /HP/i ){
	$sybasedir = "/opt/jConnect-4_2/classes";
	$oraclecheckfile = "lib/libclntsh.sl";
    }
    elsif ( $os_type =~ /AIX/i ){
	$sybasedir = "/usr/local/jConnect-4_2/classes";
	$oraclecheckfile = "lib/libclntsh.a";
    }
    else{
	$sybasedir = "/usr/local/jConnect-4_2/classes";
	#$oraclecheckfile = "lib/libclntsh.so.1.0";
	$oraclecheckfile = "lib/libclntsh.so";
    }

    $DbHost = "";
    if ( $ENV{ORACLE_HOME} ){
	$oraclehome = $ENV{ORACLE_HOME};
    }
    else{
	$oraclehome = "/opt/app/oracle/product/8.0.6";
    }

    if ( -d $ENV{INSTHOME} ){
	$ibmhome = $ENV{INSTHOME};
    }
    elsif ( $os_type =~ /SUN/i ){
	$ibmhome = "/export/home/db2inst1";
    }
    else{
	$ibmhome = "/home/db2inst1";
#	$ibmhome = "/opt/IBMdb2/V5.0";
    }

#NJR    $doUpgrade = "no";
#NJR
    $doUpgrade = "yes";
    $runSmInit = "yes";
    $restrictserver = "yes";
    $adminuser = "";
#NJR    $adminpasswd = "";
#NJR
    $adminpasswd = "admin";

    #$threadtype = "native";
    $httport = "8080";
    $serviceport = "54890";

    @deloldfiles = qw(
	    bin/AgConvertDb.sh
	    bin/AgMDUpgrade.sh
	    bin/AgFixDb.sh
	    bin/AgRenameDb.sh
	    );

    $DEFAULT_WE_MAX_SRV_THREAD	= "50";
    $DEFAULT_WE_MIN_SRV_THREAD	= "7";
    $DEFAULT_EE_MAX_SRV_THREAD	= "250";
    $DEFAULT_EE_MIN_SRV_THREAD	= "10";

    if ( -f "$srcdir/inst/SilverDispatcher" &&
	-f "$srcdir/inst/SilverCacheMgr" &&
	-f "$srcdir/inst/SilverLoadMgr")
    {
	$maxsrvthrd = $DEFAULT_EE_MAX_SRV_THREAD;
	$minsrvthrd = $DEFAULT_EE_MIN_SRV_THREAD;
    }
    else{
	$maxsrvthrd = $DEFAULT_WE_MAX_SRV_THREAD;
	$minsrvthrd = $DEFAULT_WE_MIN_SRV_THREAD;
    }

    $err_untar = $err_sminit = $err_lic = 0;
}
###########################################################################
sub ParseCmdLine{

    my $k = shift;
    my $cwd = shift;
    my $srcdir = shift;

    if ( $k ne $invokekey ) {
	my $name = $0;
	$name =~ s@.*/@@; #basename
	$name =~ s@\.pl@@; #remove extention
	$name = $name . ".sh";
	die " Please do not execute this perl script directly.\n".
	    " Execute the shell script in the same directory as\n".
	    " this script. ie: $name\n";
    }

    foreach $_ (@_){
	m@^-q@ && do {
print "NJR:" ;
print $_;
print ":NJR:";
print $quiet;
print ":NJR";
	    $quiet = $_;
	    next;
	};

	m@^-|^/[h?]$|^/help$@ && do {
	    die "$usage";
	};

	$installdir = $_;
    }

    return ($installdir, $srcdir, $cwd);
}

###########################################################################
# figure out what product type is being installed
#
sub GetProductType{
    my $srcdir = shift;
    my $file = $licensecksum;
    my $typefile;
    my $prodtype;

    if ( ! -f $file ){
	return "";
    }

    open (CKFILE, $file) or do {
	return "";
    };
    @fdata = <CKFILE>;
    close(CKFILE);

    my ($a1, $a2, $a3) = split(/\s+/, $fdata[0]);
    my ($b1, $b2, $b3) = split(/\s+/, $fdata[1]);
    my ($c1, $c2, $c3) = split(/\s+/, $fdata[2]);

    $cksumline = qx`$cksum $srcdir/$licensecmd`;
    my ($num1, $num2, $f) = split(/\s+/, $cksumline);

    if ( $num1 == $a1 && $num2 == $a2 ) {
	$typefile = $a3;
    }
    elsif ( $num1 == $b1 && $num2 == $b2 ) {
	$typefile = $b3;
    }
    elsif ( $num1 == $c1 && $num2 == $c2 ) {
	$typefile = $c3;
    }

    if ($typefile =~ /SDP/i){
	$prodtype = "SDE";
    }
    elsif ($typefile =~ /GDP/i){
	$prodtype = "SWE";
    }
    elsif ($typefile =~ /APS/i){
	$prodtype = "SEE";
    }
    else{
	if ( -f "$srcdir/inst/SilverCacheMgr" ){
	    $prodtype = "SEE";
	}
	else{
	    $prodtype = "SWE";
	}
    }

    return $prodtype;

}

###########################################################################
# sets a lot of globals
#
sub AskQuestions{

    my $licenseagreement = shift;

    my $header = "-=" x 34 . "\n".
	"\tWelcome to the $product_name installation for Unix\n\n".
	"\tPlease read the Installation Notes in:\n".
	"\t$relnotes\n".
	"-=" x 34 . "\n";
    my $go;
    my $lastlabel;

#NJR    print "\n$header\n";

    &CheckPatches($srcdir);
#NJR    &DisplayLicenseAgreement($licenseagreement);

  LABEL_INSTALLDIR:
    $installdir = &GetInstallDir($installdir);
    &ReadInstallDefaultsFile("$installdir/$installdefaults");
    $restrictserver = "yes"; # in case this is in aginstdefs (Carol S)
    $go = "";

  LABEL_ASKDOCS:
    $installdocs = $installdocs_default unless $installdocs;
    if ( -f "$srcdir/$doctarfile" || -f "$srcdir/$doctarfile.Z") {
	$mesg = "Install the SilverStream documentation files? (y/n)";
#NJR	$go = &GetAnswer($mesg, \$installdocs, $YESNO);
    }
    goto LABEL_INSTALLDIR if $go eq $BACK;

  LABEL_ASKSAMPLES:
    $installsamples = $installsamples_default unless $installsamples;
    if ( -d "$srcdir/Samples") {
	$mesg = "Install the SilverStream Sample application files? (y/n)";
#NJR	$go = &GetAnswer($mesg, \$installsamples, $YESNO);
    }
    goto LABEL_ASKDOCS if $go eq $BACK;

    my $dbansw = $dbtype unless $dbansw;
  LABEL_DBTYPE:
    $mesg = "Please enter the Database type: ".
	    "[choose option # or shortcut answer in ()] \n".
	    " 1. Informix Cloudscape                                 (C,1)\n".
	    " 2. Informix Online Dynamic Server                      (I,2)\n".
	    " 3. -not used-                                           \n".
	    " 4. Microsoft SQL Server 2000 type 4 JDBC driver        (M,4)\n".
	    " 5. IBM DB2 UDB 5.2                                     (D,5)\n".
	    " 6. IBM DB2 UDB 6.1/7.1                                 (B,6)\n".
	    " 7. Oracle 7                                            (R,7)\n".
	    " 8. Oracle 8                                            (O,8)\n".
	    " 9. Oracle 9i                                           (9)\n".
	    "10. -not used-                                           \n".
	    "11. Sybase Adaptive Server Enterprise 11                (S,11)\n".
	    "12. Sybase Adaptive Server Enterprise 12                (Y,12)\n".
	    "13. Sybase Adaptive Server Anywhere 6 with jConnect 5.x (A6,13)\n".
	    "14. Sybase Adaptive Server Anywhere 7 with jConnect 5.x (A7,14)\n".
	    "15. Sybase Adaptive Server Anywhere 8 with jConnect 5.x (A8,15)\n";

#NJR    $go = &GetAnswer($mesg, \$dbansw);
    goto LABEL_ASKSAMPLES if $go eq $BACK;

    if ( $dbansw =~ /11|^S$/i ){
	print "\n + Database Type: Sybase Adaptive Server Enterprise 11\n";
	$dbtype = "Sybase11";
    }
    elsif ( $dbansw =~ /12|^Y$/i ){
	print "\n + Database Type: Sybase Adaptive Server Enterprise 12\n";
	$dbtype = "Sybase12";
    }
    elsif ( $dbansw =~ /^13|^A6|ASA6/i ){
	print "\n + Database Type: Sybase Adaptive Server Anywhere 6 \n";
	$dbtype = "SybaseASA6";
    }
    elsif ( $dbansw =~ /^14|^A7|ASA7/i ){
	print "\n + Database Type: Sybase Adaptive Server Anywhere 7 \n";
	$dbtype = "SybaseASA7";
    }
    elsif ( $dbansw =~ /^15|^A8|ASA8/i ){
	print "\n + Database Type: Sybase Adaptive Server Anywhere 8 \n";
	$dbtype = "SybaseASA8";
    }
    elsif ( $dbansw =~ /^1|^C|cloud/i ){
	print "\n + Database Type: Informix Cloudscape\n";
	$dbtype = "Cloudscape";
    }
    elsif ( $dbansw =~ /^2|^I$|inf/i ){
	print "\n + Database Type: Informix Online Dynamic Server\n";
	$dbtype = "Informix";
    }
    elsif ( $dbansw =~ /^5|^Ib|^D|5\.2/i ){
	print "\n + Database Type: IBM DB2 UDB 5.2\n";
	$dbtype = "IBM DB2 5.2";
    }
    elsif ( $dbansw =~ /^6|^B|6\.1|7\.1/i ){
	print "\n + Database Type: IBM DB2 UDB 6.1/7.1\n";
	$dbtype = "IBM DB2 6.1/7.1";
    }
    elsif ( $dbansw =~ /^4|^M|Micro|SQL/i ){
	print "\n + Database Type: Microsoft SQL Server 2000 type 4 JDBC driver\n";
	$dbtype = "Microsoft SQL server 2000 type 4 JDBC driver";
    }
    elsif ( $dbansw =~ /^O|^R|orac|7|8|9/i ){
	if ( $dbansw =~ /7|^R/i ){
	    print "\n + Database Type: Oracle 7\n";
	    $dbtype = "Oracle7";
	}
	elsif ( $dbansw =~ /9/i ){
	    print "\n + Database Type: Oracle 9i\n";
	    $dbtype = "Oracle9";
	}
	else{
	    print "\n + Database Type: Oracle 8\n";
	    $dbtype = "Oracle8";
	}
    }
    else{
	print " (No option matched. Using the fallback default)\n";
	print "\n + Database Type: Oracle 8\n";
	$dbtype = "Oracle8";
    }

    if ( $os_type =~ /AIX/i  && $dbtype =~ /Cloudscape/i ){
	$ENV{JAVA_COMPILER} = "NONE";
    }
    else{
	$ENV{JAVA_COMPILER} = "";
    }

    if ( $dbtype =~ /Cloudscape/i ){

      LABEL_CLOUD_SMASTERPATH:
	$cloudscapeSmPath = $masterName unless $cloudscapeSmPath;

	$mesg = "----------------------------------------------------------------\n".
	    "Please enter the path to the Informix Cloudscape SilverMaster\n".
	    "database.  This can be an absolute path or relative to\n".
	    "    \"$installdir/db\"\n".
	    "If the directory specified does not exist, a new Informix Cloudscape\n".
	    "SilverMaster database will be created in that location.\n\n".
	    "Please enter the path of your SilverMaster database.";
#NJR	$go = &GetAnswer($mesg, \$cloudscapeSmPath);
	goto LABEL_DBTYPE if $go eq $BACK;

	$masterName = $cloudscapeSmPath;
	$masterName =~ s@.*/@@; #basename

	print "\n";
	my $testdir;
	if ( $cloudscapeSmPath =~ m-^/-){
	    $testdir = $cloudscapeSmPath;
	}
	else{
	    $testdir = "$installdir/db/$cloudscapeSmPath";
	}
	if ( -e $testdir ){
	    $newcloud = 0;
	    print " + Use existing Informix Cloudscape DB: $testdir\n";
	    print "   (found: $testdir)\n";
	}
	else{
	    $newcloud = 1;
	    print " + Create new Informix Cloudscape DB: $testdir\n";
	    print "   (did not find: $testdir)\n";
	}

      LABEL_CLOUDUSERPASS:
	$go = &AskUserAndPass($go);
	goto LABEL_CLOUD_SMASTERPATH if $go eq $BACK;

      LABEL_CLOUDSCAPEPATH:
	$cloudsdir = "$installdir/lib/Cloudscape" unless $cloudsdir;
	$mesg = "Please enter the path name for the Cloudscape\n".
		"directory which contains the Cloudscape jars.";
	$go = &GetAnswer($mesg, \$cloudsdir, $DIRECTORY);
	goto LABEL_CLOUDUSERPASS if $go eq $BACK;

	$lastlabel = "LABEL_CLOUDSCAPEPATH";
    }
    elsif ( $dbtype =~ /Sybase/i ){

     LABEL_SYB_JCONTYPE:
	if($dbtype =~ /Sybase11/ || $dbtype =~ /Sybase12/){
	    $jcontype = "5" unless $jcontype;
	    $mesg = "Would you like to connect with jConnect4 or jConnect5.".
		    "  (enter '4' or '5')";
	    $go = &GetAnswer($mesg, \$jcontype, "4", "5",
						"jconnect4", "jconnect5",
						"jConnect4", "jConnect5");
	    if($dbtype =~ /Sybase11/){
		if ($jcontype =~ /4/){
		    $dbtype = "Sybase11J4";
		}
		else{
		    $dbtype = "Sybase11J5";
		}
	    }
	    else{
		if ($jcontype =~ /4/){
		    $dbtype = "Sybase12J4";
		}
		else{
		    $dbtype = "Sybase12J5";
		}
	    }
	}
	goto LABEL_DBTYPE if $go eq $BACK;

      LABEL_SYB_SMASTER:
	$mesg = "Please enter the name of your SilverMaster database.";
#NJR	$go = &GetAnswer($mesg, \$masterName);
	goto LABEL_SYB_JCONTYPE if $go eq $BACK;

      LABEL_SYBUSERPASS:
	$go = &AskUserAndPass($go);
	goto LABEL_SYB_SMASTER if $go eq $BACK;

      LABEL_SYBHOST:
	$mesg = "Please enter the name of your Sybase Host.";
	$go = &GetAnswer($mesg, \$DbHost);
	goto LABEL_SYBUSERPASS if $go eq $BACK;

      LABEL_SYBPORT:
	unless ($DbPort){
	    if ($dbtype =~ /asa/i){
		$DbPort = $SYBASEASA_PORT;
	    }
	    else{
		$DbPort = $SYBASE_PORT;
	    }
	}
	$mesg = "Please enter the name of your Sybase Port.";
	$go = &GetAnswer($mesg, \$DbPort, $INTEGER);
	goto LABEL_SYBHOST if $go eq $BACK;

      LABEL_SYBCLS:
	$mesg = "Please enter the path name for the Sybase jConnect ".
		"classes directory.";
	$go = &GetAnswer($mesg, \$sybasedir, $DIRECTORY);
	goto LABEL_SYBPORT if $go eq $BACK;
	$lastlabel = LABEL_SYBCLS;

	if ( ! -f "$sybasedir/$sybjconn2jar" ){
	    $mesg = &RequiredFileMesg("$sybasedir/$sybjconn2jar", "Sybase");
	    $go = &GetAnswer($mesg, "", $BLANK);
	    goto LABEL_SYBCLS if $go eq $BACK;
	}
    }
    elsif ( $dbtype =~ /Informix/i ){

      LABEL_IFX_SMASTER:
	$mesg = "Please enter the name of your SilverMaster database.";
	$go = &GetAnswer($mesg, \$masterName);
	goto LABEL_DBTYPE if $go eq $BACK;

      LABEL_IFXUSERPASS:
	$go = &AskUserAndPass($go);
	goto LABEL_IFX_SMASTER if $go eq $BACK;

      LABEL_IFXHOST:
	$mesg = "Please enter the name of your Informix Host.";
	$go = &GetAnswer($mesg, \$DbHost);
	goto LABEL_IFXUSERPASS if $go eq $BACK;

      LABEL_IFXSERVER:
	$mesg = "Please enter the name of your Informix Server.";
	$go = &GetAnswer($mesg, \$DbServerName);
	goto LABEL_IFXHOST if $go eq $BACK;

      LABEL_IFXPORT:
	$DbPort = $INFORMIX_PORT unless $DbPort;
	$mesg = "Please enter the name of your Informix Port.";
	$go = &GetAnswer($mesg, \$DbPort, $INTEGER);
	goto LABEL_IFXSERVER if $go eq $BACK;

      LABEL_IFXCLS:
	$mesg = "Please enter the directory for your Informix JDBC driver.";
	$go = &GetAnswer($mesg, \$informixdir, $DIRECTORY);
	goto LABEL_IFXPORT if $go eq $BACK;
	$ENV{CLASSPATH} = "$informixdir/$informixjar";
	$lastlabel = LABEL_IFXCLS;

	if ( ! -f "$informixdir/$informixjar" ){
	    $mesg = &RequiredFileMesg("$informixdir/$informixjar", "Informix");
	    $go = &GetAnswer($mesg, "", $BLANK);
	    goto LABEL_IFXCLS if $go eq $BACK;
	}
    }
    elsif ( $dbtype =~ /Oracle/i ){

      LABEL_ORACTHIN:
	if ( $dbtype eq "Oracle8" ){
	    $mesg = "Please select the Oracle database driver type.\n".
		    "  1. SilverStream Oracle Driver\n".
		    "  2. Oracle Thin JDBC driver\n";
#NJR	    $go = &GetAnswer($mesg, \$oracthin, 1, 2);
	}
	elsif ( $dbtype eq "Oracle9" ) {
	    # either thin OR OCI
	    $mesg = "Please specify whether you would like to use ".
		    "the Oracle Thin JDBC driver\n".
		    "or the Oracle Call Interface (OCI).\n".
		    "  1. Oracle Call Interface\n".
		    "  2. Oracle Thin JDBC driver\n";

#NJR	    $go = &GetAnswer($mesg, \$oracthin, 1, 2);
	}
	goto LABEL_DBTYPE if $go eq $BACK;

      LABEL_ORACHOME:
	if ( $oracthin == 1 ){ # not thin, 1=oci,2=thin
	    $mesg = "Please enter the ORACLE_HOME directory.";
#NJR	    $go = &GetAnswer($mesg, \$oraclehome, $DIRECTORY);
	    goto LABEL_ORACTHIN if $go eq $BACK;
	    $ENV{ORACLE_HOME} = $oraclehome;
	    $ENV{TNS_ADMIN} = $oraclehome;
	    $orac_zip_a_dir = "$oraclehome/$orac_zip_a_subpath";

	    if ( ! -e "$oraclehome/$oraclecheckfile" ){
		$mesg = &RequiredFileMesg("$oraclehome/$oraclecheckfile",
						"Oracle");
#NJR		$go = &GetAnswer($mesg, "", $BLANK);
		goto LABEL_ORACHOME if $go eq $BACK;
	    }
	}
	goto LABEL_ORACTHIN if $go eq $BACK;

      LABEL_ORAC_ZIP_A:
	if ( $oracthin == 2 ){ # thin, not OCI, 1=oci,2=thin
	    $mesg = "Please enter the path where $ORAC_ZIP_A.jar ".
		    "(or $ORAC_ZIP_A.zip)\ncan be located.";
	    $orac_zip_a_dir = $oraclehome unless $orac_zip_a_dir;
	    $go = &GetAnswer($mesg, \$orac_zip_a_dir, $DIRECTORY);
	    goto LABEL_ORACTHIN if $go eq $BACK;
	    $oraclehome = $orac_zip_a_dir;
	    $ENV{ORACLE_HOME} = $oraclehome;

	    unless ( -e "$orac_zip_a_dir/$ORAC_ZIP_A.jar" ||
		     -e "$orac_zip_a_dir/$ORAC_ZIP_A.zip" )
	    {
		$mesg = &RequiredFileMesg("$orac_zip_a_dir/$ORAC_ZIP_A.jar (or .zip)",
						"Oracle");
		$go = &GetAnswer($mesg, "", $BLANK);
		goto LABEL_ORAC_ZIP_A if $go eq $BACK;
	    }
	}
	goto LABEL_ORACHOME if $go eq $BACK;

      LABEL_ORAC_ZIP_B:
	if ( $oracthin == 2 ){ # thin, 1=oci,2=thin
	    #$orac_zip_b_dir = $orac_zip_a_dir unless $orac_zip_b_dir;
	    $mesg = "Please enter the path where $ORAC_ZIP_B.jar ".
		    "(or $ORAC_ZIP_B.zip)\ncan be located (optional).";
	    $go = &GetAnswer($mesg, \$orac_zip_b_dir, $BLANK);
	    if ( ! -d $orac_zip_b_dir ){
		print "\n".
		    " ** Warning: directory $orac_zip_b_dir was not found.\n".
		    "    value will not be used...\n";
		    $orac_zip_b_dir = "";
	    }
	}
	goto LABEL_ORAC_ZIP_A if $go eq $BACK;

       LABEL_ORACSYN:
	if ( $dbtype eq "Oracle8" || $dbtype eq "Oracle9" ) {
	    $mesg = "Would you like to use Oracle synonym support";
#NJR	    $go = &GetAnswer($mesg, \$oracsyn, $YESNO);
	}
	elsif ( $dbtype eq "Oracle9" &&  $oracthin == 2){ # 2 = thin, 1 = oci
	    $mesg = "Would you like to use Oracle synonym support";
	    $go = &GetAnswer($mesg, \$oracsyn, $YESNO);
	}
	goto LABEL_ORAC_ZIP_B if $go eq $BACK;

       LABEL_ORACTHINURL1:
	if ( $oracthin == 2 ){ # if thin and not oracle7, 1=oci,2=thin
	    $mesg = "Please enter the Oracle database server hostname.";
	    $go = &GetAnswer($mesg, \$oracsrv);
	}
	goto LABEL_ORACSYN if $go eq $BACK;

       LABEL_ORACTHINURL2:
	if ( $oracthin == 2 ){ # if thin and not oracle7, 1=oci,2=thin
	    $oracport = $URL_ORACLE_PORT unless $oracport;
	    $mesg = "Please enter the Oracle port number.";
	    $go = &GetAnswer($mesg, \$oracport);
	}
	goto LABEL_ORACTHINURL1 if $go eq $BACK;

       LABEL_ORACTHINURL3:
	if ( $oracthin == 2 ){ # if thin and not oracle7, 1=oci,2=thin
    	    $oracsid = $URL_ORACLE_SID unless $oracsid;
	    $mesg = "Please enter the Oracle DB instance ID (SID).";
	    $go = &GetAnswer($mesg, \$oracsid);
	}
	goto LABEL_ORACTHINURL2 if $go eq $BACK;

       LABEL_ORACTNS:
	if ( $oracthin == 1 ){ # if oralcle 8 and not thin, or oracle 7
	    $mesg = "Please enter the Oracle TNS name.";
#NJR	    $go = &GetAnswer($mesg, \$oractns);
	}
	goto LABEL_ORACTHINURL3 if $go eq $BACK;

      LABEL_ORACDBALIAS:
	$mesg = "Please enter the name of your SilverMaster database.";
#NJR	$go = &GetAnswer($mesg, \$masterName);
	goto LABEL_ORACTNS if $go eq $BACK;
	$DbHost = $masterName;

      LABEL_ORACUSERPASS:
#NJR	$go = &AskUserAndPass($go);
	goto LABEL_ORACDBALIAS if $go eq $BACK;

	$lastlabel = LABEL_ORACUSERPASS;
    }
    elsif ( $dbtype =~ /IBM/i ){

      LABEL_DB2HOME:
	$mesg = "Please enter the path for the IBM DB2 instance.";
	$go = &GetAnswer($mesg, \$ibmhome, $DIRECTORY);
	goto LABEL_DBTYPE if $go eq $BACK;

	if ( ! -f "$ibmhome/$ibmcheckfile" ){
	    $mesg = &RequiredFileMesg("$ibmhome/$ibmcheckfile", "IBM DB2");
	    $go = &GetAnswer($mesg, "", $BLANK);
	    goto LABEL_DB2HOME if $go eq $BACK;
	}

      LABEL_IBMMASTER:
	$mesg = "Please enter the name of your SilverMaster database.";
	$go = &GetAnswer($mesg, \$masterName);
	goto LABEL_DB2HOME if $go eq $BACK;

      LABEL_IBMUSERPASS:
	$go = &AskUserAndPass($go);
	goto LABEL_IBMMASTER if $go eq $BACK;

	$lastlabel = LABEL_IBMUSERPASS;
    }
    elsif ( $dbtype =~ /Microsoft/i ){

      LABEL_MSMASTER:
	$mesg = "Please enter the name of your SilverMaster database.";
	$go = &GetAnswer($mesg, \$masterName);
	goto LABEL_DBTYPE if $go eq $BACK;

      LABEL_MSUSERPASS:
	$go = &AskUserAndPass($go);
	goto LABEL_MSMASTER if $go eq $BACK;

      LABEL_MSHOST:
	$mesg = "Please enter the name of your Microsoft SQL Host.";
	$go = &GetAnswer($mesg, \$DbHost);
	goto LABEL_MSUSERPASS if $go eq $BACK;

      LABEL_MSPORT:
	$DbPort = $MSSQL_PORT unless $DbPort;
	$mesg = "Please enter the Microsoft SQL port number.";
	$go = &GetAnswer($mesg, \$DbPort, $INTEGER);
	goto LABEL_MSHOST if $go eq $BACK;

      LABEL_MSDIR:
	$mesg = "Please enter the directory that contains the\n".
		"Microsoft SQL Server 2000 type 4 JDBC driver jar files.";
	$go = &GetAnswer($mesg, \$msdir, $DIRECTORY);
	goto LABEL_MSPORT if $go eq $BACK;

	$lastlabel = LABEL_MSDIR;
    }

  LABEL_UPGRADE:
    if ($updateassurance){
	$doUpgrade = "yes";
    }
    else {
	$mesg = "Upgrade your existing SilverMaster? ". #davea 2/19
		"(y/n) ";
#NJR	$go = &GetAnswer($mesg, \$doUpgrade, $YESNO);
    }
    goto $lastlabel if $go eq $BACK;

  LABEL_CONFIG_SRV:
    if ( $doUpgrade eq "yes"){
	$go = &GetOldRegistryFile($installdir, $registryfile, $go);
    }
    else{
	$mesg = "Should this installation Configure your SilverStream Server\n".
	       " (execute SilverMasterInit)? (y/n) ";
#NJR	$go = &GetAnswer($mesg, \$runSmInit, $YESNO);
    }
    goto LABEL_UPGRADE if $go eq $BACK;

  LABEL_RESTRICTSRV:
    if ( $runSmInit eq "yes" && $doUpgrade eq "no"){
	$mesg = "Restrict access to the SilverStream server?";
#NJR	$go = &GetAnswer($mesg, \$restrictserver, $YESNO);
    }
    goto LABEL_CONFIG_SRV if $go eq $BACK;

  LABEL_ADMINUSER:
    if ( $runSmInit eq "yes" && $doUpgrade eq "no"){
	$mesg = "Enter the user name for the SilverStream ".
		"administration account.";
#NJR	$go = &GetAnswer($mesg, \$adminuser);
    }
    goto LABEL_RESTRICTSRV if $go eq $BACK;

  LABEL_ADMINPASSWD:
    if ( $runSmInit eq "yes" && $doUpgrade eq "no"){
	$mesg = "Enter the password for the SilverStream ".
		"administration account.";
#NJR	$go = &GetAnswer($mesg, \$adminpasswd);
    }
    goto LABEL_ADMINUSER if $go eq $BACK;

  LABEL_HPORT:
#NJR    $go = &GetHttpPort();
    goto LABEL_ADMINPASSWD if $go eq $BACK;

  LABEL_SVCPORT:
    if ( $runSmInit eq "yes"){
	$mesg = "Please enter the Name Service port number.";
#NJR	$go = &GetAnswer($mesg, \$serviceport);
    }
    goto LABEL_HPORT if $go eq $BACK;

  LABEL_CONFIG_JBMQ:
#NJR    print "\n".
#NJR    "----------------------------------------------------------------------\n".
#NJR    "Would you like to configure jBroker MQ at the end of this install?\n".
#NJR    "The configuration tool starts an X Windows client.\n".
#NJR    "If your environment is not set up to run an X Windows client,\n".
#NJR    "then please choose 'no' for a silent install of jBroker MQ files.\n".
#NJR    "If you choose 'no' you can still configure jBroker MQ later.\n".
#NJR    "Your current DISPLAY is set to the following.\n".
#NJR    "         DISPLAY=$ENV{DISPLAY}\n";
    $mesg = "Would you like to configure jBroker MQ at the end of this install?.";
#NJR    $go = &GetAnswer($mesg, \$configjbmq, $YESNO);
    goto LABEL_SVCPORT if $go eq $BACK;


  LABEL_LICSTR:
    $mesg = "Please enter the SilverStream License string.";
#NJR    $go = &GetAnswer($mesg, \$licenseStr);
    goto LABEL_CONFIG_JBMQ if $go eq $BACK;

    $ENV{SILVERSTREAMROOT} = $installdir;

    #$ENV{JRE_HOME} = "$installdir/jre";
    $ENV{JRE_HOME} = "\$SILVERSTREAMROOT/jre";

    #$ENV{JBROKER_HOME} = "$installdir/jbroker";
    #$ENV{JBROKER3_HOME} = "\$SILVERSTREAMROOT/jbroker";

    &SetLdLibPath();

  LABEL_CONFIRM:
    $go = &Confirmation($installdir);
    goto LABEL_LICSTR if $go eq $BACK;

    &WriteInstallDefaultsFile("$installdir/$installdefaults");
}

###########################################################################
sub RequiredFileMesg{

    my $file = shift;
    my $product = shift;

    my $mesg = "  ***********************************************\n".
	"  Unable to find the required file: \n\n".
	"    $file\n\n".
	"  The SilverStream server may not start due to this problem,\n".
	"  consult the SilverStream documentation for details on\n".
	"  configuring $product for use as a SilverMaster database.\n".
	"  You may now continue or quit this install?\n".
	"  (just press 'enter' to continue) \n";

    return $mesg;
}
###########################################################################
# this just takes a file and assign variables in this script
sub ReadInstallDefaultsFile
{
    $file = shift;
    if ( ! -f $file ){
	return;
    }

    open(IFILE, "$file") or return;
    print " + reading $file\n";

    while(<IFILE>)
    {
        s/#.*//; # clean any comments
        s/"//g;  # get rid of quotes
        s/^[\s\$]*//; # clean out any leading spaces or $'s
        m@([^\s=]*)\s*[\s=]?\s*(.*)\s*@;
        if ( $1 && $2)
        {
            my $cmd = "\$$1 = \"$2\"";
            print " + $cmd\n";
            eval $cmd;
        }
    }
    close(IFILE);
}
###########################################################################
sub WriteInstallDefaultsFile{

    $file = shift;

    open(IFILE, ">$file") or return;
    print " + writing $file\n";

    print IFILE "# SilverStream re-installation defaults file\n";
    print IFILE "# This is a generated file.,\n";
    print IFILE "# DO NOT EDIT THIS FILE.\n";
    print IFILE "dbtype = $dbtype\n"			if $dbtype;
    print IFILE "masterName = $masterName\n"		if $masterName;
    print IFILE "dbUsername = $dbUsername\n"		if $dbUsername;
    print IFILE "DbHost = $DbHost\n"			if $DbHost;
    print IFILE "DbPort = $DbPort\n"			if $DbPort;
    print IFILE "sybasedir = $sybasedir\n"		if $sybasedir;
    print IFILE "msdir = $msdir\n"			if $msdir;
    print IFILE "cloudsdir = $cloudsdir\n"		if $cloudsdir;
    print IFILE "DbServerName = $DbServerName\n"	if $DbServerName;
    print IFILE "informixdir = $informixdir\n"		if $informixdir;
    print IFILE "oraclehome = $oraclehome\n"		if $oraclehome;
    print IFILE "ibmhome = $ibmhome\n"			if $ibmhome;
    print IFILE "serviceport = $serviceport\n"		if $serviceport;
    print IFILE "httport = $httport\n"			if $httport;
    print IFILE "licenseStr = $licenseStr\n"		if $licenseStr;
    print IFILE "installdocs = $installdocs\n"		if $installdocs;
    print IFILE "installsamples = $installsamples\n"	if $installsamples;
    print IFILE "oracthin = $oracthin\n"		if $oracthin;
    print IFILE "oracsyn = $oracsyn\n"			if $oracsyn;
    print IFILE "oracsrv = $oracsrv\n"			if $oracsrv;
    print IFILE "oracport = $oracport\n"		if $oracport;
    print IFILE "oracsid = $oracsid\n"			if $oracsid;
    print IFILE "oractns = $oractns\n"			if $oractns;
    print IFILE "adminuser = $adminuser\n"		if $adminuser;
    print IFILE "orac_zip_a_dir = $orac_zip_a_dir\n"	if $orac_zip_a_dir;
    print IFILE "orac_zip_b_dir = $orac_zip_b_dir\n"	if $orac_zip_b_dir;

    close(IFILE);
}
###########################################################################
sub CopyEEfiles{

    my $src = shift;
    my $dest = shift;

    my @files = qw(
	    SilverCacheMgr
	    SilverCacheMgr_
	    SilverDispatcher
	    SilverDispatcher_
	    SilverLoadMgr
	    SilverLoadMgr_
	);

    foreach $f (@files){
	if ( -f "$src/inst/$f" ){
	    print qq' + $cp "$src/inst/$f" "$dest/bin/$f"\n';
	    system(qq'$cp "$src/inst/$f" "$dest/bin/$f"');
	}
    }
}
###########################################################################
sub GetOldRegistryFile{

    my $installdir = shift;
    my $registryfile = shift;
    my $go = shift;
    my $previousdir = $installdir;

  LABEL_GETPREVIOUSDIR:
    while( ! -f "$installdir/$registryfile"){
	print "\n\tIn order to upgrade, the SilverStream registry file ".
		"must first be located.\n".
		"\t  $previousdir/$registryfile\n".
		"\twas not found.\n";
	$mesg = "Enter the directory name that contains the previous ".
		"version of SilverStream.";
	$go = &GetAnswer($mesg, \$previousdir, $DIRECTORY);
	return $go if $go eq $BACK;

	if ( -f "$previousdir/$registryfile"){
	    print "$cp $previousdir/$registryfile $installdir\n";
	    system(qq'$cp "$previousdir/$registryfile" "$installdir"');
	}
	else{
	    redo;
	}

	unless ( -f "$installdir/$registryfile"){
	    print " ** Error: $installdir/$registryfile: does not exist\n";
	    print "    Copy must have failed.\n";
	}
    }
    return $go;
}
###########################################################################
#
#
sub DisplayLicenseAgreement{

    my $licenseagreement = shift;

    print "Press return to view the license agreement: ";
    my $answ = <STDIN>;
    $answ =~ /^q/i && die "Exiting...\n";
    print "\n" . "-" x 60 . "\n";

    if ( -f $licenseagreement ){
	system("$more $licenseagreement");
    }
    else{
	print "\n  **Warning: License file not found: $licenseagreement\n";
    }
    print "\n" . "-" x 60 . "\n\n";

    print qq'Please Enter "y" to accept this license agreement,\n'.
	    qq'or "n" to decline (y/n) [n] : ';
    $answ = <STDIN>;
    $answ !~ /^y/i && die "Exiting...\n";
    print "\n" . "-" x 60 . "\n\n";
}

###########################################################################
# example scripts to start the server after a reboot
sub WriteStartupScripts{

    my $installdir = shift;
    my $os_type = shift;
    my $save_msg = "\n";

    my $filename1;
    my $hp_linkname;
    if ( $os_type =~ /HP/i ){
	$filename1 = "$installdir/sbin/init.d/silverserver";
	$filename2 = "$installdir/sbin/init.d/silvercluster";
	$hp_linkname1 = "$installdir/sbin/rc3.d/S300silverserver";
	$hp_linkname2 = "$installdir/sbin/rc3.d/S301silvercluster";
	system("$mkdir -p $installdir/sbin/rc3.d");
	system("$mkdir -p $installdir/sbin/init.d");
    }
    elsif ( $os_type =~ /SUN/i ){
	$filename1 = "$installdir/etc/rc3.d/S90silverserver";
	$filename2 = "$installdir/etc/rc3.d/S91silvercluster";
	system("$mkdir -p $installdir/etc/rc3.d");
    }
    elsif ( $os_type =~ /AIX/i ){
	system("$mkdir -p $installdir/etc");
	$filename1 = "$installdir/etc/rc.silverserver";
	$filename2 = "$installdir/etc/rc.silvercluster";
    }
    else{
	return;
    }
    $save_msg .= "  - $filename1\n".
		 "  - $filename2\n";

#    unless ( $os_type =~ /SUN/i ){
#	# this does not seem to work on HP at this time.
#	return;
#    }

#    print "\nStart the SilverServer automatically after each reboot ? ".
#	    "(y/n) [y] : ";
#    chomp(my $answ = <STDIN>);
#    $answ = "y" unless $answ;
#    if ($answ !~ /y/i){
#	return;
#    }

    ###########################################################
    # Create the server script file text here
    #

    print " + writing sample startup script $filename1\n";

    $scriptext = "#!$shell\n".
	qq'#\n'.
	qq'    SILVERSTREAMROOT="$installdir"\n'.
        qq'    silverserver="\$SILVERSTREAMROOT/bin/SilverServer"\n'.
	qq'    PATH="/sbin:/usr/sbin:/usr/bin:\$SILVERSTREAMROOT/jre/bin"\n'.
	qq'    export PATH\n';

    $scriptext .= <<'EOF';

    if [ ! -f "$silverserver" ] ; then
	exit
    fi

    killproc() {		# kill the named process(es)
	    pid=`/usr/bin/ps -ef |
		/usr/bin/grep $1 |
		/usr/bin/sed -e 's/^  *[^ ]* *//' -e 's/ .*//'`
	    [ "$pid" != "" ] && /usr/bin/kill $pid
    }

    #
    # Start/stop the server
    #

    case "$1" in
	start_msg)
		    echo "Start SilverStream server" ;;
	stop_msg)
		    echo "Stop SilverStream server" ;;
	start)
		    $silverserver & ;;
	stop)
		    killproc SilverServer ;;	# kill daemon
    esac
EOF
    #
    # end of the script text
    ####################################################################

    open(RCFILE, ">$filename1") or do {
	warn "\n ** Error: could not open RC file: $filename1: $!\n";
	#print "  (no 'root' permissions?)\n";
	return;
	};
    print RCFILE $scriptext;
    close(RCFILE);
    chmod 0755, $filename1;

    ###########################################################
    # Create the cluster script file text here
    #

    print " + writing sample startup script $filename2\n";

    $scriptext = "#!$shell\n".
	qq'#\n'.
	qq'    SILVERSTREAMROOT="$installdir"\n'.
        qq'    silvercachemgr="\$SILVERSTREAMROOT/bin/SilverCacheMgr"\n'.
        qq'    silverdispatcher="\$SILVERSTREAMROOT/bin/SilverDispatcher"\n'.
        qq'    silverloadmgr="\$SILVERSTREAMROOT/bin/SilverLoadMgr"\n'.
	qq'    PATH="/sbin:/usr/sbin:/usr/bin:\$SILVERSTREAMROOT/jre/bin"\n'.
	qq'    export PATH\n';

    $scriptext .= <<'EOF';

    if [ ! -f "$silvercachemgr" ] ; then
	exit
    fi
    if [ ! -f "$silverdispatcher" ] ; then
	exit
    fi
    if [ ! -f "$silverloadmgr" ] ; then
	exit
    fi

    killproc() {		# kill the named process(es)
	    pid=`/usr/bin/ps -ef |
		/usr/bin/grep $1 |
		/usr/bin/sed -e 's/^  *[^ ]* *//' -e 's/ .*//'`
	    [ "$pid" != "" ] && /usr/bin/kill $pid
    }

    #
    # Start/stop the cluster tools
    #

    case "$1" in
	start_msg)
		    echo "Start SilverStream cluster tools" ;;
	stop_msg)
		    echo "Stop SilverStream cluster tools" ;;
	start)
		    $silvercachemgr &
		    $silverdispatcher &
		    $silverloadmgr & ;;
	stop)
		    killproc SilverLoadMgr
		    killproc SilverDispatcher
		    killproc SilverCacheMgr ;;
    esac
EOF
    #
    # end of the script text
    ####################################################################

    open(RCFILE, ">$filename2") or do {
	warn "\n ** Error: could not open RC file: $filename2: $!\n";
	#print "  (no 'root' permissions?)\n";
	return;
	};
    print RCFILE $scriptext;
    close(RCFILE);
    chmod 0755, $filename2;
    
    if ( $os_type =~ /HP/i ){
	unlink("$hp_linkname1"); #remove first just to make sure.
	symlink($filename1, $hp_linkname1) or do {
	    warn "\n ** Error: could not create symlink ".
		    "$filename1 -> $hp_linkname1: $!\n";
	    };
	unlink("$hp_linkname2"); #remove first just to make sure.
	symlink($filename2, $hp_linkname2) or do {
	    warn "\n ** Error: could not create symlink ".
		    "$filename2 -> $hp_linkname2: $!\n";
	    };
    }

    my $msg = "  These sample 'startup on reboot' scripts can be ".
		"installed on your system \n".
	    "   by your system administrator.\n";
    print "$msg";
    $save_msg .= $msg;
    return "$save_msg";
}

###########################################################################
sub StartServer{

    my $installdir = shift;

#NJR    print "\nStart the SilverServer now ? (y/n) [y] : ";
#NJR    chomp(my $answ = <STDIN>);
    $answ = "y" unless $answ;
    if ($answ !~ /y/i){
	return;
    }

    print " + cd $installdir\n";
    chdir($installdir);
    print " + $installdir/bin/SilverServer &\n";
    system("$installdir/bin/SilverServer &");
}

###########################################################################
# globals $dbUsername, $dbPassword
#
sub AskUserAndPass{

    my $go = shift;

    goto LABEL_GETPASS if $go eq $BACK;

  LABEL_GETUSER:
    $mesg = "Please enter the Database access username.";
    $go = &GetAnswer($mesg, \$dbUsername);
    return $BACK if $go eq $BACK;

  LABEL_GETPASS:
    $mesg = "Please enter the Database access password.";
    $go = &GetAnswer($mesg, \$dbPassword);
    goto LABEL_GETUSER if $go eq $BACK;

    return wantarray ? ($go, $dbUsername,$dbPassword) : $go;
}
###########################################################################
sub Confirmation{

    my $installdir = shift;

 print "\n
    -----------------------------------------------------------------------
    Install Notes    : $relnotes
    SilverMaster Name: $masterName
    DB Username      : $dbUsername
    DB Password      : $dbPassword
    License string   : $licenseStr
    HTTP Port number : $httport";
 if ( $dbtype =~ /Sybase/i ){
    print "
    Sybase Host      : $DbHost
    Sybase Port      : $DbPort\n
    CLASSPATH = $ENV{CLASSPATH}";
 }
 elsif ( $dbtype =~ /Informix/i ){
    print "
    Informix Host    : $DbHost
    Informix Port    : $DbPort\n
    CLASSPATH = $ENV{CLASSPATH}";
 }
 elsif ( $dbtype =~ /Microsoft/i ){
    print "
    MS SQL Host      : $DbHost
    MS SQL Port      : $DbPort\n";
 }
 elsif ( $dbtype =~ /Oracle/i ){
    print "
    Oracle Db Alias       : $DbHost
    ORACLE_HOME           : $ENV{ORACLE_HOME}";
 }
 elsif ( $dbtype =~ /Cloudscape/i ){
    print "
    Cloudscape path  : $cloudsdir";
 }

 print "\n
    $libpathev = $ENV{$libpathev}
";

 chdir($installdir) or gracefulexit(10, " ** Error: Can not chdir to $installdir: $!\n");

 unless ($quiet){
    print "\nInstall the $product_name\n".
	"into '$installdir' now? (y/n/b=back) [y] : ";
    chomp(my $answ = <STDIN>);
    print "\n";
    $answ = "y" unless $answ;
    $answ =~ /^b/i && return $BACK;
    $answ !~ /^y/i && die "Exiting...\n";
 }

}

###########################################################################
sub GetInstallDir{

    my $installdir = shift;
    my $go;

    LOOP_ASKDIR:
    {
	unless($quiet){
	    $mesg = "Please enter the directory where you would like to ".
		"install\nthe $product_name.";
	    $go = &GetAnswer($mesg, \$installdir);
	    if ( $go eq $BACK ) {
		print "\n ** Can not go back from here **\n";
		redo LOOP_ASKDIR;
	    }
	}

	unless ( $installdir =~ m@^/@ ) {
	    print "\n *** prepending current working directory to the ".
		    "relative pathname entered:\n".
		    "        $cwd  +  $installdir\n";
	    $installdir = "$cwd/$installdir";
	}

	print "\n";
	if ( -d $installdir ) {
	    print " + directory exists: $installdir\n";
	}
	else{
	    my $answ;
	    unless($quiet){
		print " + directory does not exist: $installdir\n";
		print "mkdir $installdir ? (y/n) [y]: ";
		chomp($answ = <STDIN>);
	    }
	    $answ = "y" unless $answ;
	    $answ =~ /^q/i && die "Exiting...\n";
	    $answ !~ /^y/i && redo LOOP_ASKDIR;

	    print "\n";
	    system("$mkdir -p $installdir/tmp");
	    system("$mkdir -p $installdir/db");
	    unless( -d $installdir ) {
		&Perr(" ** Error: install directory not found: $installdir.");
		redo LOOP_ASKDIR;
	    }
	}
	return $installdir;
    } #LOOP_ASKDIR
}

###########################################################################
sub WriteSpecialFile{

    my $installdir = shift;

    unless ( $os_type =~ /HP/i ){
	return;
    }

    system("/usr/bin/uuidgen -o $installdir/bin/.agUUID");
}

###########################################################################
sub CopyExportDomestic{

    my $srcd = shift;
    my $instdir = shift;

    my $sji = "SilverJRunnerInstall";
    my $sje = "$sji.exe";
    my $sjs = "$sji.sh";
    my $sjsol = "${sji}Solaris.sh";
    my $sjlnx = "${sji}Linux.sh";
    my $sjd = "Resources/$sji";

    my $sd_dir = "lib";

    print " + $cp $srcd/inst/*crypto* $instdir/bin\n";
    system(qq'$cp $srcd/inst/*crypto* "$instdir/bin"');

    if ( -f "$srcd/$sjd/$sje" ){
	print " + $cp $srcd/$sjd/$sje $instdir/$sjd/$sje\n";
	system(qq'$cp $srcd/$sjd/$sje "$instdir/$sjd/$sje"');
    }
    if ( -f "$srcd/$sjd/$sjs" ){
	print " + $cp $srcd/$sjd/$sjs $instdir/$sjd/$sjs\n";
	system(qq'$cp $srcd/$sjd/$sjs "$instdir/$sjd/$sjs"');
    }
    if ( -f "$srcd/$sjd/$sjsol" ){
	print " + $cp $srcd/$sjd/$sjs $instdir/$sjd/$sjsol\n";
	system(qq'$cp $srcd/$sjd/$sjsol "$instdir/$sjd/$sjsol"');
    }
    if ( -f "$srcd/$sjd/$sjlnx" ){
	print " + $cp $srcd/$sjd/$sjlnx $instdir/$sjd/$sjlnx\n";
	system(qq'$cp $srcd/$sjd/$sjlnx "$instdir/$sjd/$sjlnx"');
    }
    if ( -f "$srcd/$sjd/exteNdToolsWindows.exe" ){
        print " + $cp $srcd/$sjd/exteNdToolsWindows.exe instdir/$sjd/exteNdToolsWindows.exe\n";
        system(qq'$cp $srcd/$sjd/exteNdToolsWindows.exe "$instdir/$sjd/exteNdToolsWindows.exe"')
    }

}

###########################################################################
sub CopyFilesToJre{

    my $srcdir = shift;
    my $destdir = shift;

    chdir($destdir) or
	gracefulexit(10, " ** Error: Can not chdir to $destdir: $!\n");

    my $jbbin;
    my $jrebin;
    if ( $os_type =~ /SUN/i ){
	if ($hardware_type =~ /sun/i ){
	    $jbbin = "solaris";
	    $jrebin = "sparc";
	}
	else{ #solaris intel
	    $jbbin = "solaris-x86";
	    $jrebin = "x86";
	}
    }
    elsif ( $os_type =~ /AIX/i ){
	$jbbin = "aix";
	$jrebin = "ppc";
    }
    elsif ( $os_type =~ /Linux/i ){
	$jbbin = "linux-rh";
	$jrebin = "i386";
    }
    elsif ( $os_type =~ /HP/i ){
	$jbbin = "hp-ux";
	$jrebin = "PA_RISC";
    }

    system("$mkdir -p jre/lib/ext/classes");
    system("$mkdir jre/lib/ext/$jrebin");

    unlink("jre/lib/ext/jbroker.jar");
    unlink("jre/lib/ext/tools.jar");
    unlink("jre/lib/ext/classes/orb-lite.properties");

    system("$cp jbroker/lib/jorbd.properties jre/lib/ext/classes");
    system("$cp jbroker/lib/nameserver.properties jre/lib/ext/classes");
    system("$cp jbroker/lib/poa.properties jre/lib/ext/classes");
    system("$cp jbroker/lib/security.properties jre/lib/ext/classes");
    system("$cp jbroker/lib/transactions.properties jre/lib/ext/classes");

    system("$cp jbroker/lib/orb.properties jre/lib/ext/classes");
    system("$cp jbroker/lib/orb.properties jre/lib");

    system("$cp jbroker/lib/jbroker-rt.jar jre/lib/ext");
    system("$cp jbroker/lib/jbroker-tools.jar jre/lib/ext");
    system("$cp jbroker/lib/jbroker-ssl.jar jre/lib/ext");

    system("$cp $srcdir/jre/jbroker-tm.jar jre/lib/ext");
    system("$cp $srcdir/jre/jbroker.licenses jre/lib/ext/classes");

    system("$cp jbroker/bin/$jbbin/libjbroker* jre/lib/ext/$jrebin");

    $JB_PLATFORM_DIR = $jbbin; # global
}

###########################################################################
sub LinkDocs{

    my $src = shift;
    my $topdir = shift;
    my $relnotes = "relnotes.html";

    my %linktolist = (
	"$relnotes"		=> "Docs",
        );
	#"InstallGuideTOC.html" 	=> "Docs/help/books",

    foreach $l (keys %linktolist){
	if ( -f "$topdir/$linktolist{$l}/$l" && ! -e "$topdir/$l" ){
	    print " + link $linktolist{$l}/$l\n";
	    symlink ("$linktolist{$l}/$l", "$topdir/$l");
	}
    }

    print " + $cp $src/Docs/$relnotes $topdir/Docs/$relnotes\n";
    system(qq'$cp $src/Docs/$relnotes "$topdir/Docs/$relnotes"');

}

###########################################################################
sub DeleteFiles{

    $installdir = shift;
    $filelist = shift;

    foreach $f ( @$filelist ){
	if ( -f "$installdir/$f"){
	    print " + rm $installdir/$f\n";
	    unlink("$installdir/$f")
	}
    }

}

###########################################################################
sub UnPackFile{

    my $srcdir = shift;
    my @tars = @_;
    my $tarzfile;
    my $rtn = 0;

    foreach $tarfile ( @tars ){

	$tarfile = "$srcdir/$tarfile";

	if ( -f $tarfile ){
	    $cmd = "$tar $taropt $tarfile";
	}
	elsif ( -f "$tarfile.z"){
	    $tarzfile = "$tarfile.z";
	    $cmd = "$zcat < $tarzfile | $tar $taropt -";
	}
	elsif ( -f "$tarfile.Z"){
	    $tarzfile = "$tarfile.Z";
	    $cmd = "$zcat < $tarzfile | $tar $taropt -";
	}
	else{
	    gracefulexit(53, " ** Error: tar data file was not found: $tarfile* \n");
	}

	print "\n + $cmd\n\n";
	open(UNPACK, "$cmd 2>&1 |") or
	    gracefulexit(10, " ** Error: could not execute: $cmd :$!\n");

	while (<UNPACK>){
	    print " +";
	    print;
	}
	close(UNPACK);

	if ( $? ) {
	    gracefulexit(10, " ** Error: $cmd failed. returned status: $?\n");
#RW	    print "<press <return> to continue or 'q' to quit>";
#RW	    chomp(my $answ = <STDIN>);
#RW	    $answ =~ /^q/i && die "Exiting...\n";
#RW	    print "\n";
#RW	    $rtn += $?;
	}
	print "-" x 70 . "\n\n";
    }

    return $rtn;
}

###########################################################################
sub CopyOracleLib{

    my $agdir = shift;
    my $orac = shift;

    my $dest    = "$agdir/bin/$AG_ORACLELIB";
    my $src8  = "$agdir/bin/$AG_ORACLELIB8";
    my $src8i = "$agdir/bin/$AG_ORACLELIB8i";

    my $ol8 = "$orac/lib/$ORACLELIB8";
    my $ol8i = "$orac/lib/$ORACLELIB8i";


    #
    # only for Oracle on Solaris
    #
    if ($dbtype =~ /Oracle9/i){
	return;
    }
    elsif ($dbtype !~ /Oracle/i){
	return;
    }
    elsif ($os_type !~ /SUN/i ){
	return;
    }

    #
    # check for the source files that will be copied
    #
    if ( ! -f $src8 ){
	gracefulexit(53, "  ** Internal media error: $src8 was not found\n");
#RW	print "<press <return> to continue>";
#RW	my $answ = <STDIN>;
#RW	return;
    }
    elsif ( ! -f $src8i ){
    gracefulexit(53, "  ** Internal media error: $src8i was not found\n");
#RW	print "<press <return> to continue>";
#RW	my $answ = <STDIN>;
#RW	return;
    }

    #
    # copy the correct file depending on whether Oracle8 or Oracle8i
    # is installed.
    #
    if ( -f $ol8 ){
	print " + $cp $src8 $dest\n";
	system(qq'$cp "$src8" "$dest"');
    }
    elsif ( -f $ol8i ){
	print " + $cp $src8i $dest\n";
	system(qq'$cp "$src8i" "$dest"');
    }
    else{
	gracefulexit(53, " ** Error: Oracle library file was not found.\n".
		"    Could not find either:\n".
		"      $ol8\n".
		"  or\n".
		"      $ol8i\n");
#RW	print "<press <return> to continue>";
#RW	my $answ = <STDIN>;
    }
}


###########################################################################
# globals used: httport, masterName, dbtype
#
sub EditHttpdPropFile{

    my $installdir = shift;
    my $justUpdateMasterName = shift;

    my $propFile = "$installdir/$PROPS_PATH/$HPROPS_FILENAME";
    my $dboot = "$installdir/dbBoot";
    my ($line, $new);

    print " + Editing: $propFile\n";

    my @filedata = &EditFile("READ", $propFile);

    $line = "http-server.com.sssw.srv.http.webmaster=";
    $new = $line . $masterName;
    @filedata = &EditFile($line, $new, @filedata);

    if ($justUpdateMasterName) {
	&EditFile("WRITE", $propFile, @filedata);
	return;
    }

    $line = "http-server.com.sssw.srv.ContentCache.Disk.directory=";
    $new = $line . "$installdir/tmp/$CACHE_DISKDIR";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.updatelicenses=";
    $new = "";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "Add Unlimited Server Licenses";
    $new = "";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.client.debug=";
    $new = $line . "0";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.agent.debug=";
    $new = $line . "0";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.sql.debug=";
    $new = $line . "0";
    @filedata = &EditFile($line, $new, @filedata);

    # stuff to skip when just configuring the DBs, not installing
    #if (bDoAll){
    #$line = "http-server.com.sssw.srv.port=";
    $line = "http-server.com.sssw.srv.port_rt=";
    $new = $line . $httport;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.port_des=";
    $new = $line . $httport;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.port_admin=";
    $new = $line . $httport;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.orb.orbplatforms=";
    $new = $line . "$installdir/$PROPS_PATH/orbs.dbl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.sminit=";
    $new = $line . "$installdir/$PROPS_PATH/sminit.props";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.nameServicePort=";
    $new = $line . $serviceport;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.jmsServerLaunch=";
    if ($configjbmq eq "yes") {
	$new = $line . "true";
    }
    else{
	$new = $line . "false";
    }
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.enable.fulcrum=";
    if ( -d "$installdir/fts" ){
	$new = $line . "true";
    }
    else{
	$new = $line . "false";
    }
    @filedata = &EditFile($line, $new, @filedata);

    #//////// Upgrade special //////////

    $line = "http-server.com.sssw.srv.autoupgrade.25=";
    $new = $line . "FALSE";
    @filedata = &EditFile($line, $new, @filedata);

    #//////// Microsoft SQL Server / Sybase Special //////////

    $line = "http-server.DB.Driver=";
    $new = "#" . $line;
    @filedata = &EditFile($line, $new, @filedata);

    #//////// JDBC / Sybase special /////////

    $line = "http-server.Jdbc.Driver=";
    $new = "#" . $line;
    @filedata = &EditFile($line, $new, @filedata);

    my ($jdbcurl, $jdbcurlattr) = &SetJdbcUrl($dbtype, $installdir);

    $line = "http-server.Jdbc.URL=";
    if ($jdbcurl){
	$new = $line . $jdbcurl;
    }
    else{
	$new = "#" . $line;
    }
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.Jdbc.URL.Attributes=";
    if ($jdbcurlattr){
	$new = $line . $jdbcurlattr;
    }
    else{
	$new = "#" . $line;
    }
    @filedata = &EditFile($line, $new, @filedata);

    #//////// old Oracle Special //////////
    $line = "http-server.com.sssw.dbboot.db.oracle.tablenamespace=";
    $new = $line;
    $new = "#" . $new;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.db.dbplatforms=";
    $new = $line . "$installdir/$PROPS_PATH/platforms.dbl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.Jdbc.LDSKey=";
    my $ldskey = &SetLdsKey($dbtype);
    $new = $line . $ldskey;
    @filedata = &EditFile($line, $new, @filedata);

    &EditFile("WRITE", $propFile, @filedata);
}

###########################################################################
sub SetLdsKey{

    my $dbtype = shift;
    my $ldskey;

    if ( $dbtype =~ /Oracle7/i ){
	if ($oractns eq $masterName){
	    $ldskey = $LDSKEY_ORACLE7;
	}
	else{
	    $ldskey = $LDSKEY_ORACLE7_CUST;
	}
    }
    elsif ( $dbtype =~ /Oracle8/i ){
	if ($oracthin == 2){ # 2 = thin, 1=oci,2=thin
	    if ($oracsyn eq "yes"){
		$ldskey = $LDSKEY_ORACLE8_TSQI;
	    }
	    else{
		$ldskey = $LDSKEY_ORACLE8_THIN;
	    }
	}
	else{ # not thin
	    if ($oracsyn eq "yes"){
		$ldskey = $LDSKEY_ORACLE8_SQI;
	    }
	    elsif ($oractns eq $masterName){
		$ldskey = $LDSKEY_ORACLE8;
	    }
	    else{
		$ldskey = $LDSKEY_ORACLE8_CUST;
	    }
	}
    }
    elsif ( $dbtype =~ /Oracle9/i ){
	if ($oracthin == 2){ # 2 = thin, 1 = oci
	    if ($oracsyn eq "yes"){
		$ldskey = $LDSKEY_ORACLE9_TSQI;
	    }
	    else{
		$ldskey = $LDSKEY_ORACLE9_THIN;
	    }
	}
	else{
#	    if ($oracsyn eq "yes"){
#		$ldskey = $LDSKEY_ORACLE9_OCI_SYN;
#	    }
#	    else{
		$ldskey = $LDSKEY_ORACLE9_OCI;
#	    }
	}
    }
    elsif ( $dbtype =~ /SybaseASA6/i ) {
	$ldskey = $LDSKEY_SYB_ASA6_JCON;
    }
    elsif ( $dbtype =~ /SybaseASA7/i ) {
	$ldskey = $LDSKEY_SYB_ASA7_JCON;
    }
    elsif ( $dbtype =~ /SybaseASA8/i ) {
	$ldskey = $LDSKEY_SYB_ASA8_JCON;
    }
    elsif ( $dbtype =~ /Sybase11J4/i ) {
	$ldskey = $LDSKEY_SYBASE_SYS_JCON4;
    }
    elsif ( $dbtype =~ /Sybase11J5/i ) {
	$ldskey = $LDSKEY_SYBASE_SYS_JCON5;
    }
    elsif ( $dbtype =~ /Sybase12J4/i ) {
	$ldskey = $LDSKEY_SYBASE_SYS12_JCON4;
    }
    elsif ( $dbtype =~ /Sybase12J5/i ) {
	$ldskey = $LDSKEY_SYBASE_SYS12_JCON5;
    }
    elsif ( $dbtype =~ /Informix/i ){
	$ldskey = $LDSKEY_INFORMIX;
    }
    elsif ( $dbtype =~ /Ibm.*5/i ){
	$ldskey = $LDSKEY_IBM_DB2_5;
    }
    elsif ( $dbtype =~ /Ibm.*6/i ){
	$ldskey = $LDSKEY_IBM_DB2_6;
    }
    elsif ( $dbtype =~ /Cloudscape/i ){
	$ldskey = $LDSKEY_CLOUDSCAPE;
    }
    elsif ( $dbtype =~ /Microsoft/i ){
	$ldskey = $LDSKEY_MS_SQL2k_MS;
    }
    else{
	print " ** Internal script error: dbtype = $dbtype !\n";
    }

    return $ldskey;
}

###########################################################################
sub SetJdbcUrl{

    my $dbtype = shift;
    my $instdir = shift;
    my $jdbcurl;
    my $jdbcurlattr;

    if ( $dbtype =~ /Oracle7/i ){
	if ($oractns eq $masterName){
	    $jdbcurl = $ORACLE_JDBC_URL . $masterName;
	}
	else{
	    $jdbcurl = $ORACLE_JDBC_URL . $oractns;
	}
    }
    elsif ( $dbtype =~ /Oracle8/i ){
	if ($oracthin == 2){ # 1=oci,2=thin
	    $jdbcurl = $ORACLE_THIN_URL . "@" . $oracsrv .":".
			$oracport .":". $oracsid;
	}
	else{ # not thin
	    if ($oracsyn eq "yes"){
		$jdbcurl = $ORACLE_JDBC_URL . $oractns;
	    }
	    elsif ($oractns eq $masterName){
		$jdbcurl = $ORACLE_JDBC_URL . $masterName;
	    }
	    else{
		$jdbcurl = $ORACLE_JDBC_URL . $oractns;
	    }
	}
    }
    elsif ( $dbtype =~ /Oracle9/i ){
	if ($oracthin == 2){ # 2 = thin, 1 = oci
	    $jdbcurl = $ORACLE_THIN_URL . "@" . $oracsrv .":".
			$oracport .":". $oracsid;
	}
	else{
	    $jdbcurl = $ORACLE_OCI_URL . "@" . $oractns;
	}
    }
    elsif ( $dbtype =~ /SybaseASA/i ) {
	$jdbcurl = $SYBASE_JDBC_URL . $DbHost .":". $DbPort .
		"/?SERVICENAME=".  $masterName;
    }
    elsif ( $dbtype =~ /Sybase/i ) {
	$jdbcurl = $SYBASE_JDBC_URL . $DbHost .":". $DbPort ."/".
		$masterName;
    }
    elsif ( $dbtype =~ /Ibm/i ){
	$jdbcurl = $IBM_DB2_URL . $masterName;
    }
    elsif ( $dbtype =~ /Informix/i ) {
	$jdbcurl = $INFORMIX_JDBC_URL . $DbHost .":". $DbPort ."/".
	    $masterName .":INFORMIXSERVER=". $DbServerName .
	    ";user=".  $dbUsername .";password=". $dbPassword .";";
    }
    elsif ( $dbtype =~ /Cloudscape/i ) {
	$jdbcurl = $CLOUDSCAPE_URL . $cloudscapeSmPath;
	if ( $newcloud){
	    $jdbcurlattr = "create=true";
	}
	else{
	    $jdbcurlattr = "";
	}
    }
    elsif ( $dbtype =~ /Microsoft/i ) {
	$jdbcurl = $MS_SQL2k_URL ."//". $DbHost .":". $DbPort;
    }
    else{
	$jdbcurl = "";
    }

    return ($jdbcurl, $jdbcurlattr);
}

###########################################################################
# globals used: httport, masterName, dbtype, maxsrvthrd, minsrvthrd
#
sub EditSminitPropFile{

    my $installdir = shift;

    my $propFile = "$installdir/$PROPS_PATH/$SPROPS_FILENAME";
    my $dboot = "$installdir/dbBoot";
    my ($line, $new);

    print " + Editing: $propFile\n";

    my @filedata = &EditFile("READ", $propFile);

    $line = "http-server.com.sssw.srv.nameServicePort=";
    $new = $line . $serviceport;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.ContentCache.enable=";
    $new = $line . $CACHE_ENABLE;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.ContentCache.Memory.maxContentSize=";
    $new = $line . $CACHE_MEMCONTSIZE;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.ContentCache.Memory.maxSize=";
    $new = $line . $CACHE_MEMSIZE;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.ContentCache.Disk.maxContentSize=";
    $new = $line . $CACHE_DISKCONTSIZE;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.ContentCache.Disk.maxSize=";
    $new = $line . $CACHE_DISKSIZE;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.db.baseschema=";
    $new = $line . "$dboot/BaseSchema.sdl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.db.dbprofiles=";
    $new = $line . "$dboot/Silvermaster.pdl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.db.sqlgenschema=";
    $new = $line . "$dboot/Silvermaster.sdl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.resrc.masterresources=";
    $new = $line . "$dboot/Silvermaster.rdl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.resrc.baseresources=";
    $new = $line . "$dboot/BaseResources.rdl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.resrc.rootdir=";
    $new = $line . "$installdir";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.updatelicenses=";
    $new = "";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "Add Unlimited Server Licenses";
    $new = "";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.db.viewLenient=";
    $new = $line . "true";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.http.ClientPool.maxThreads=";
    $new = $line . $maxsrvthrd; #DEFAULT_EE_MAX_SRV_THREAD;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.http.ClientPool.maxIdle=";
    $new = $line . $maxsrvthrd; #DEFAULT_EE_MAX_SRV_THREAD;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.ambry.Result.Buffers=";
    $new = $line . $maxsrvthrd; #DEFAULT_EE_MAX_SRV_THREAD;
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.srv.http.ClientPool.minFree=";
    $new = $line . $minsrvthrd; #DEFAULT_EE_MIN_SRV_THREAD
    @filedata = &EditFile($line, $new, @filedata);

    #//////// Upgrade special //////////

    $line = "http-server.com.sssw.dbboot.db.masterschema=";
    $new = $line . "$dboot/Silvermaster.sdl";
    @filedata = &EditFile($line, $new, @filedata);

    $line = "http-server.com.sssw.dbboot.db.droptablesfile=";
    $new = $line . "$dboot/Silvermaster.sdl";
    @filedata = &EditFile($line, $new, @filedata);

    &EditFile("WRITE", $propFile, @filedata);
}

###########################################################################
# globals used: httport
#
sub EditSmcPropsFile{

    my $installdir = shift;

    my $propFile = "$installdir/$PROPS_PATH/$SMCPROPS_FILENAME";
    my ($line, $new);

    print " + Editing: $propFile\n";

    my @filedata = &EditFile("READ", $propFile);

    $line = "server.names=";
    $new = $line . "localhost:" . $httport;
    @filedata = &EditFile($line, $new, @filedata);

    &EditFile("WRITE", $propFile, @filedata);
}

###########################################################################
sub EditFile{

    my $key = shift;
    my $newstr = shift; # or file name to open or write
    my @filedata = @_;

    if ( $key eq "READ" ){
	open(READFILE, "$newstr") or
	    gracefulexit(10, " ** Error: could not open prop file to read: $newstr: $!\n");
	@filedata = <READFILE>;
	close(READFILE);
	for $_ (@filedata){
	    s/\015$//;	# remove the <control-M>
	}
	return @filedata;
    }
    elsif ( $key eq "WRITE" ){
	open(PROPFILE, ">$newstr") or
	    gracefulexit(10, " ** Error: could not open prop file to write: $newstr: $!\n");
	print PROPFILE @filedata;
	close(PROPFILE);
	return;
    }

    # reverse and double all the slashes
    #$newstr =~ s-^/-\\\\-;
    #$newstr =~ s-(.)[/\\]+-$1\\\\-g;

    # all only single forward slashes
    $newstr =~ s-^[\\/]+-/-;
    $newstr .= "\n";

    for $_ (@filedata){
	if ( m@^$key@ || m@^#+\s*$key@){
	    $_ = $newstr;
	    return @filedata;
	}
    }

    # else if not found, add it
    push (@filedata, $newstr);
    return @filedata;
}

###########################################################################
# return the global $httport
#
sub GetHttpPort {

    my $answ;

    print "\n".
    "  -------------------------------------------------------------------\n".
    "  Due to privilege restrictions on UNIX for ports below 1024,\n".
    "  the SilverStream Application Server defaults to using port 8080.\n".
    "\n".
    "  If you wish to run the SilverStream Server on port 80, or any\n".
    "  other port less than 1024, the SilverServer process must be\n".
    "  started by the user root.\n".
    "\n".
    "  However, for any port other than 80 you need to specify the port\n".
    "  number when you add the server to the SilverStream Designer or\n".
    "  when you specify URLs for that server.\n".
    "\n".
    "  For example, in the SilverStream Designer, select the\n".
    "  'Choose Server icon' and enter the name as follows:\n".
    "  Hostname:port# (e.g. myserver:8080)\n".
    "\n".
    "  The same servername:portnumber specification should be used for\n".
    "  application URLs on the server, such as:\n".
    "\n".
    "  http://myserver1:8080/SilverStream/Pages/SilverClient.html\n";

    $mesg = "Enter the HTTP port number.";

    $answ = &GetAnswer($mesg, \$httport, $INTEGER);

    return $answ;
}

###########################################################################
sub Copydir{

    my $srcparent = shift;
    my $instdir = shift;
    my $srcd = shift;

    if ( ! -d $srcparent ) {
	gracefulexit(53, " ** Error: source parent directory '$srcparent' does not exist\n");
#RW	&Pause();
#RW	return;
    }
    elsif ( ! -d $instdir ) {
	gracefulexit(53, " ** Error: install directory '$instdir' does not exist\n");
#RW	&Pause();
#RW	return;
    }
    elsif ( ! -d "$srcparent/$srcd" ) {
	gracefulexit(53, " ** Error: source directory '$srcd' does not exist\n");
#RW	&Pause();
#RW	return;
    }

    system(qq'$cp -r "$srcparent/$srcd" "$instdir"');
}

###########################################################################
sub Pause{

    my $msg = shift;
    my $x;

    $msg = "press 'Enter' to continue" unless $msg;

    print "\n<<< $msg (q=quit) >>>";
    $x = <STDIN>;
    die "Exiting...\n" if $x =~ m@^q$@i;
    return $x;
}
###########################################################################
sub CheckPatches{

    my $un = `$uname -sr`;
    my @needpatchlist;
    my @havepatchlist;
    my %needpat;
    #my %havepat; # global for now
    my $cddir = shift;
    my $readme;

    if ($hardware_type =~ /sun/i ){
	$readme = "README.sparc";
	if ($un =~ /Sun.*5\.6/){
	    @needpatchlist = @sunpatches56;
	}
	elsif ($un =~ /Sun.*5\.7/){
	    @needpatchlist = @sunpatches57;
	}
	elsif ($un =~ /Sun.*5\.8/){
	    @needpatchlist = @sunpatches58;
	}
	else{
	    return;
	}
    }
    elsif ($hardware_type =~ /i86pc/i ){
	$readme = "README.i386";
	if ($un =~ /Sun.*5\.6/){
	    @needpatchlist = @solintelpatches56;
	}
	elsif ($un =~ /Sun.*5\.7/){
	    @needpatchlist = @solintelpatches57;
	}
	elsif ($un =~ /Sun.*5\.8/){
	    @needpatchlist = @solintelpatches58;
	}
	else{
	    return;
	}
    }
    else{
	return;
    }

    print "Checking to make sure all the required Solaris patches are ".
	    "installed...\n";
    # get the list of installed patches
    my @sr = `$showrev -p`;

    # split out patch number and patch version into an array
    foreach $_ (@sr){
	chomp;
	m/^Patch: (\d+)-(\d+) .*/;
	&AddHavePat($1, $2);
	m/.* Obsoletes:([ \d,-]+)Requires.*/;
	foreach $_ (split(/[ ,]/, $1)){
	    next unless $_;
	    m/(\d+)-(\d+)/;
	    &AddHavePat($1, $2);
	}
    }

    # split out patch number and patch version into an array
    print "Required patches include:\n";
    foreach $_ (@needpatchlist){
	print "    $_\n";
	m/(\d+)-(\d+)/;
	$needpat{$1} = $2;
    }
    print "\n";

    $pmsg = 0;
    foreach $npat (keys %needpat){
	my $foundpat = 0;
	my $oldver = 0;
	foreach $hpat (keys %havepat){
	    # check patch number
	    if ( $npat eq $hpat ){
		# check patch version
		if($havepat{$hpat} < $needpat{$npat}){
		    $oldver = "$hpat-$havepat{$hpat}";
		}
		else{
		    $foundpat = "found";
		}
		last;
	    }
	}
	if($oldver){
	    print " ** Warning: found patch $oldver ".
		"that needs to be updated to $npat-$needpat{$npat}\n";
	    $pmsg = "print final message";
	}
	unless($foundpat){
	    print " ** Warning: required patch $npat-$needpat{$npat} ".
		"was not detected\n";
	    $pmsg = "print final message";
	}
    }

    if($pmsg){
	    gracefulexit(12, "\n".
            "Please verify that you have installed any patches required \n".
            "by the Sun JRE.  Refer to '$readme' for patch requirements\n".
            "of the JRE at the time of shipment.\n".
            "    $cddir/Required/$readme\n");
#RW	    "Exit install now? (y/n) [y] :";
#RW	chomp(my $answ = <STDIN>);
#RW	$answ !~ /^n/i && die "Exiting...\n";
    }
    else{
	print "This system appears to have all the required patches ".
		"already installed.\n";
    }
    print "\n";
}

#########################################################################
# add patch to global hash havepat
sub AddHavePat{
    my $patnum = shift;
    my $patver = shift;

    if ( $havepat{$patnum} ){
	if ( $patver > $havepat{$patnum} ){
	    $havepat{$patnum} = $patver;
	}
    }
    else{
	$havepat{$patnum} = $patver;
    }
}

###########################################################################
sub SetLdLibPath{

    print "\n + setting up $libpathev ...\n";

    if ( $os_type =~ /SUN/i ){
	if ($hardware_type =~ /sun/i ){
	    $libpath = &AddPathElement("\$JRE_HOME/lib/sparc",
				    "begining", $libpath, "nowarn");
	}
	else{ #solaris intel
	    $libpath = &AddPathElement("\$JRE_HOME/lib/i386",
				    "begining", $libpath, "nowarn");
	}
    }
    elsif ( $os_type =~ /AIX/i ){
	$libpath = &AddPathElement("\$JRE_HOME/bin/classic",
				    "begining", $libpath, "nowarn");
	$libpath = &AddPathElement("\$JRE_HOME/bin",
				    "begining", $libpath, "nowarn");
    }
    elsif ( $os_type =~ /Linux/i ){
	$libpath = &AddPathElement("\$JRE_HOME/bin",
				    "end", $libpath, "nowarn");
	$libpath = &AddPathElement("\$JRE_HOME/lib/i386/client",
				    "end", $libpath, "nowarn");
	$libpath = &AddPathElement("\$JRE_HOME/lib/i386/native_threads",
				    "end", $libpath, "nowarn");
	$libpath = &AddPathElement("\$JRE_HOME/lib/i386",
				    "end", $libpath, "nowarn");
    }

    if ( $dbtype =~ /Oracle/i && $oracthin == 1){ # 1 = oci not thin
	$libpath = &AddPathElement("\$ORACLE_HOME/lib", "end", $libpath);
    }

    $libpath = &AddPathElement("\$SILVERSTREAMROOT/bin",
				    "end", $libpath, "nowarn");
    $libpath = &AddPathElement("\$SILVERSTREAMROOT/fts/lib",
				    "end", $libpath, "nowarn");

    #$libpath = &AddPathElement("\$SILVERSTREAMROOT/jbroker/bin/$jbbin", "end",
				    #$libpath, "nowarn");

    if ( $dbtype =~ /IBM/i  && $ibmhome ){
	$libpath = &AddPathElement("$ibmhome/sqllib/lib", "end", $libpath);
    }
    elsif ( $dbtype =~ /oracle9/i ){
	if ( $os_type =~ /AIX/i || $os_type =~ /hp/i ){
	    $libpath = &AddPathElement("\$ORACLE_HOME/lib32", "end", $libpath);
	}
    }

    $ENV{$libpathev} = $libpath;

    #print "\n + $libpathev=$ENV{$libpathev}\n";
}

###########################################################################
#
# add an element to a path if it does not already exist.
# $_[1] = place which could be begining, or end to specify where to
# add the element if needed
#
sub AddPathElement{

    my $element = shift;
    my $place = shift;
    my $path = shift;
    my @path = split(/:/, $path);
    my $nowarn = shift;

    if ( $element =~ /^\$/ ){
	# expand the environment variable and then check the dir
	my $ev = $element;
	$ev =~ s/^\$//;
	my $dir = $ev;
	if ($dir =~ m@/@){
	    $dir =~ s@[^/]*/@@;
	    $ev =~ s@/.*@@;
	}
	else{
	    $dir = "";
	}
	 unless ( -d "$ENV{$ev}/$dir" ){
	    print " ** Warning: $ENV{$ev}/$dir: does not exist\n"
							unless $nowarn;
	}
    }
    elsif ( ! -d $element ){
	print " ** Warning: $element: does not exist\n" unless $nowarn;
    }

    # don't add the same thing if it already exists
    foreach (@path) {
        if ( m@^$element$@ ){
	    print " + found : $element\n";
	    return $path;
        }
    }

    # not found so...
    unless ( $path ){
	print " + set    : $element\n";
	return "$element";
    }
    elsif ( $place =~ m@^b@ ){
	print " + insert : $element\n";
	return "$element:$path";
    }
    else{
	print " + add    : $element\n";
	return "$path:$element";
    }
}

###########################################################################
sub BackupFile{

    my $filepath = shift;

    unless ( -f $filepath ){
	return;
    }

    my ($sec,$min,$hour24,$mday,$mon,$yr,$wday,$yday,$isdst);
    ($sec,$min,$hour24,$mday,$mon,$yr,$wday,$yday,$isdst) = localtime(time);

    $yr += 1900; #y2k compliant
    $mon++; # zero based array

    my $fileNew = "${filepath}_$mon-$mday-$yr.$hour24:$min:$sec";

    print " + $cp $filepath $fileNew\n";
    system(qq'$cp "$filepath" "$fileNew"');
}

###########################################################################

# TLAN - 1/23/04
#    Added this method to get the current character encoding from the Oracle
#    database.  This method passes back a valid character encoding.  If the 
#    was a problem getting the encoding because the stored procedure is not
#    supported on the database, or an invalid login string is provided, 
#    the database character encoding defaults to WE8ISO8859P1.  This setting
#    is most likely for those Oracle systems that don't support the stored
#    procedure used in this function to get the database character encoding.

sub getDBCharEncoding  {

$oracsidtns="default";
if ( ! defined $oracsid || $oracsid eq "") {
   $oracsidtns = $oractns;
} else {
   $oracsidtns = $oracsid;
}

open FOO,"$oraclehome/bin/sqlplus -s << EOC
$dbUsername/$dbPassword\@$oracsidtns
set feedback off
set pagesize 0
select pvm_global.get_db_characterset from dual;
EOC |" || gracefulexit(32, "ERROR: could not get the database character set.");

$charencoding="default";
while (<FOO>) {
   chomp();
   $charencoding = $_;
}

print "\n + Oracle database character set encoding = ---$charencoding---\n";
if (  $charencoding ne "UTF8" && $charencoding ne "WE8ISO8859P1" ) {
   $charencoding="WE8ISO8859P1";
   print " + A valid Oracle database character set encoding could not be determined.\n";
   print " +    Character encoding has been set to the default of WE8ISO8859P1 in .agprofile.\n";
   print " +    If this is not appropriate, set the ISO_SETTING env variable to the desired\n";
   print " +    value in .agprofile.\n\n";
}

return $charencoding;
}

###########################################################################
sub WriteProfileScript{

    my $installdir = shift;

    $filename = "$installdir/$agprofile";

    &BackupFile($filename);

    print " + Writing profile script: $filename\n";
    open(SCRPT, ">$filename") or do {
	warn " ** Warning: could not open: $filename: $!\n";
	return;
    };

    print SCRPT
      "#!/bin/echo This_file_should_be_sourced,_not_executed:\n\n".
    qq'# this is the SilverStream environment profile\n\n'.
    qq'SILVERSTREAMROOT="$installdir" ; export SILVERSTREAMROOT\n\n'.
    qq'JRE_HOME=$ENV{JRE_HOME} ; export JRE_HOME\n\n';
    #qq'JBROKER3_HOME=$ENV{JBROKER3_HOME} ; export JBROKER3_HOME\n\n';

    if ( $dbtype =~ /Oracle/i ){
	print SCRPT
	qq'ORACLE_HOME=$ENV{ORACLE_HOME} ; export ORACLE_HOME\n\n';
    }

    if ( $dbtype =~ /IBM/i ){
	if ( -f "$ibmhome/sqllib/db2profile"){
	    print SCRPT " . $ibmhome/sqllib/db2profile\n\n";
	}
	else{
	    print
	    "\n***********************************************************\n".
	    "The file $ibmhome/sqllib/db2profile was not found.\n".
	    "This file sets the environment variables needed to connect\n".
	    "to DB2. You must set up your DB2 environment manually.\n".
	    "Press 'enter' to continue : ";
	    my $answ = <STDIN>;
	    print "\n";
	}
    }

    ###
    ### Set the HPproc (HP only).  This must be done before the
    ### SHLIB_PATH is set.
    ###

    if($os_type =~ /HP/i) {
        print SCRPT
        qq'HPcpu=`/usr/bin/getconf CPU_VERSION`\n'.
        qq'case "\$HPcpu" in\n'.
        qq'    532) HPproc=PA_RISC2.0 ;;\n'.
        qq'      *) HPproc=PA_RISC ;;\n'.
        qq'esac\n\n';
    }

    ###
    ### Add the Java entries to the front of the LIBPATH.
    ### Note the special case for HP (support for the +classic option).
    ### The double-square-bracket notation will work on ksh but not bash.
    ###

    $tab = "";
    if($os_type =~ /HP/i) {
        $tab = "    ";
        print SCRPT
        qq'if [[ \$* = *+classic* ]]; then\n'.
        qq'    $libpathev="\$JRE_HOME/$subPathForClassicVm:\$JRE_HOME/$subPathForCommonJavaLibs"\n'.
        qq'else\n';
    }

    print SCRPT
    qq'${tab}case \${0##*/} in\n'.
    qq'${tab}    SilverServer | SilverCacheMgr | SilverDispatcher | SilverLoadMgr)\n'.
    qq'${tab}        $libpathev="\$JRE_HOME/$subPathForHotSpotServerVm:\$JRE_HOME/$subPathForCommonJavaLibs" ;;\n'.
    qq'${tab}    *)\n'.
    qq'${tab}        $libpathev="\$JRE_HOME/$subPathForHotSpotClientVm:\$JRE_HOME/$subPathForCommonJavaLibs" ;;\n'.
    qq'${tab}esac\n';
    
    ($os_type =~ /HP/i) and print SCRPT qq'fi\n';

    print SCRPT qq'\n';

    ###
    ### Here are some additional platforms-specific LIBPATH entries
    ### and other environment variables.
    ###

    print SCRPT
    qq'$libpathev="\$$libpathev:$ENV{$libpathev}"; '.
    qq'export $libpathev\n\n';

    if ( $os_type =~ /AIX/i ){
	print SCRPT
	    "# Ensure 1->1 Kernal threading\n".
	    "AIXTHREAD_SCOPE=S;export AIXTHREAD_SCOPE\n".
	    "AIXTHREAD_MUTEX_DEBUG=OFF;export AIXTHREAD_MUTEX_DEBUG\n".
	    "AIXTHREAD_RWLOCK_DEBUG=OFF;export AIXTHREAD_RWLOCK_DEBUG\n".
	    "AIXTHREAD_COND_DEBUG=OFF;export AIXTHREAD_COND_DEBUG\n".
	    "AIXTHREAD_MNRATIO=1.1;export AIXTHREAD_MNRATIO\n\n";
	if ( $dbtype =~ /Cloudscape/i ){
	    print SCRPT
	    "JAVA_COMPILER=$ENV{JAVA_COMPILER} ; export JAVA_COMPILER\n\n";
	}
    }

    print SCRPT
    qq'# Fulcrum SearchServer\n'.
    qq'FULCREATE="\$SILVERSTREAMROOT/fts/fultext"; export FULCREATE\n'.
    qq'FULSEARCH="\$SILVERSTREAMROOT/fts/fultext"; export FULSEARCH\n'.
    qq'FULTEMP=/tmp; export FULTEMP\n\n'.
    qq'PATH="\$SILVERSTREAMROOT/fts/bin:\$JRE_HOME/bin:\$SILVERSTREAMROOT/bin:\$PATH" ; '.
    qq'export PATH \n\n'.
    qq'AGCLASSPATH="\$AGCLASSPATH:\$SILVERSTREAMROOT/fts/lib/ssjdbc10.jar"; '.
    qq'export AGCLASSPATH\n\n';

#qq'[ "\$JAVA_COMPILER" ] || JAVA_COMPILER=$ENV{JAVA_COMPILER} ; export JAVA_COMPILER\n\n'.
# qq'JAVA_HOME=$ENV{JAVA_HOME} ; export JAVA_HOME\n'.
# qq'RMI_IIOP_HOME=$ENV{RMI_IIOP_HOME} ; export RMI_IIOP_HOME\n\n'.
# qq'PATH="\$RMI_IIOP_HOME/bin:\$JAVA_HOME/bin:\$JAVA_HOME/jre/bin:\$PATH" ; '.

    if ( $dbtype =~ /Informix/i ) {
	print SCRPT
	"CLASSPATH=$ENV{CLASSPATH} ; export CLASSPATH\n\n";
    }
    elsif ( $dbtype =~ /Sybase/i ) {
	print SCRPT
	"AGCLASSPATH=\$AGCLASSPATH:$sybasedir/$sybjconn2jar\n";
    }
    elsif ( $dbtype =~ /Cloudscape/i ){
	print SCRPT
	"AGCLASSPATH=\$AGCLASSPATH:$cloudsdir/cloudscape.jar:$cloudsdir/RmiJdbc.jar:$cloudsdir/cloudutil.jar\n";
    }
    elsif ( $dbtype =~ /IBM/i ){
	print SCRPT
	"AGCLASSPATH=\$AGCLASSPATH:$ibmhome/$ibmcheckfile\n";
    }
    elsif ( $dbtype =~ /Oracle/i ){
	if ( -f "$orac_zip_a_dir/$ORAC_ZIP_A.zip" ){
	    print SCRPT
	    "AGCLASSPATH=\$AGCLASSPATH:$orac_zip_a_dir/$ORAC_ZIP_A.zip\n";
	}
	else{
	    print SCRPT
	    "AGCLASSPATH=\$AGCLASSPATH:$orac_zip_a_dir/$ORAC_ZIP_A.jar\n";
	}

	if ( -f "$orac_zip_b_dir/$ORAC_ZIP_B.jar" ){
	    print SCRPT
	    "AGCLASSPATH=\$AGCLASSPATH:$orac_zip_b_dir/$ORAC_ZIP_B.jar\n";
	}
	elsif ( -f "$orac_zip_b_dir/$ORAC_ZIP_B.zip" ){
	    print SCRPT
	    "AGCLASSPATH=\$AGCLASSPATH:$orac_zip_b_dir/$ORAC_ZIP_B.zip\n";
	}

#  TLAN - 1/23/04 - Added capability to get the current character encoding from the Oracle
#         database.  In addition, added code to generate script to be included in the
#         .agprofile that uses the character encoding value to properly check and set the
#         the NLS_LANG variable.

        # Get the oracle db character encoding and blast out the appropriate block
        # of script text to the .agprofile
        $encoding = &getDBCharEncoding();

        print SCRPT "\n\n";
        print SCRPT "# This Quallaby script validates localization and NLS_LANG settings.\n";
        print SCRPT "# It checks locale, if ISO8859 is installed, if LANG and LC_ALL are empty and\n"; 
        print SCRPT "# the other LC_* variables are \"C\",\n";
        print SCRPT "# it changes all locale variables to a default value of en_US.ISO8859-1.\n";
        print SCRPT "# If a valid locale is found, then no locale changes are made.\n";
        print SCRPT " \n";

	print SCRPT "# check if ISO8859 is installed\n";
	print SCRPT "function checkISO8859 {\n";
	print SCRPT "    mylocale=\"en_US.ISO8859-1\"\n";
	print SCRPT "    check_ISO8859=`locale -a | grep \$mylocale`\n";
	print SCRPT "    if [ \"\$check_ISO8859\" == \"\" ]; then\n";
	print SCRPT "       existISO8859=1\n";
	print SCRPT "       echo \"***********************************\"\n";
	print SCRPT "	    echo \"\$mylocale not installed!\"\n";
	print SCRPT "       echo \"***********************************\"\n";
	print SCRPT "    else\n";
	print SCRPT "       existISO8859=0\n";
	print SCRPT "    fi\n";
	print SCRPT "    return \$existISO8859\n";
	print SCRPT "} #end checkISO8859\n";

        print SCRPT "# run locale and check for a match\n";
        print SCRPT "function checklocale {\n";
        print SCRPT "   localematch=0\n";
        print SCRPT "   locale=`locale | xargs`\n";
        print SCRPT "   [[ \$locale = *LC_CTYPE=+(C|\"C\"|[[:space:]]L)* &&\n";
        print SCRPT "   \$locale = *LC_NUMERIC=+(C|\"C\"|[[:space:]]L)* &&\n";
        print SCRPT "   \$locale = *LC_TIME=+(C|\"C\"|[[:space:]]L)* &&\n";
        print SCRPT "   \$locale = *LC_COLLATE=+(C|\"C\"|[[:space:]]L)* &&\n";
        print SCRPT "   \$locale = *LC_MONETARY=+(C|\"C\"|[[:space:]]L)* &&\n";
        print SCRPT "   \$locale = *LC_MESSAGES=+(C|\"C\"|[[:space:]]L)* ]] &&\n";
        print SCRPT "   localematch=1\n";
        print SCRPT "   return \$localematch\n";
        print SCRPT "} #end checklocale\n";

        print SCRPT "# change to new locale\n";
        print SCRPT "function changelocale {\n";
        print SCRPT "   newlocale=\$1\n";
        print SCRPT "   LC_ALL=\$newlocale\n";
        print SCRPT "   LANG=\$newlocale\n";
        print SCRPT "   export LC_ALL\n";
        print SCRPT "   export LANG\n";
        print SCRPT "} #end changelocale\n";

        print SCRPT "# MAIN\n";

        print SCRPT "newlocale=\"en_US.ISO8859-1\"\n";

        print SCRPT "checklocale\n";
        print SCRPT "result=\$?\n";
        print SCRPT "if [ \$result = 1 ]; then\n";

        print SCRPT "   # Check if ISO8859 is installed\n";
        print SCRPT "   checkISO8859\n";
        print SCRPT "   supportISO8859=$?\n";
        print SCRPT "   if [ \$supportISO8859 = 0 ]; then\n";

        print SCRPT "      changelocale \$newlocale\n";
        print SCRPT "      checklocale\n";
        print SCRPT "      dblcheck=\$?\n";
        print SCRPT "      if [ \$dblcheck = 1 ]; then\n";
        print SCRPT "           echo \"Locale change failed, locale still set to default C or blank\"\n";
        print SCRPT "      fi\n";
        print SCRPT "   fi\n";
        print SCRPT "fi\n";

        print SCRPT "\n";
        print SCRPT "# If NLS_LANG is not defined, define it with a default\n";
        print SCRPT "# Check if ISO_SETTING is set correctly, change it if needed.\n";

        print SCRPT "\n";
        print SCRPT "ISO_SETTING=\"$encoding\"\n";

        print SCRPT "\n";
        print SCRPT "# Default to WE8ISO8859P1 if we get an \"Error\" or an Oracle exception string, or it is not one of the two\n";
        print SCRPT "# possible chactacter encodings below.\n";
        print SCRPT "if [ \"\$ISO_SETTING\" != \"UTF8\" -a \"\$ISO_SETTING\" != \"WE8ISO8859P1\"  ]; then\n";
        print SCRPT "   ISO_SETTING=\"WE8ISO8859P1\" \n";
        print SCRPT "   DEFAULT_NLS_LANG=\"AMERICAN_AMERICA.WE8ISO8859P1\"\n";
        print SCRPT "else\n";
        print SCRPT "   DEFAULT_NLS_LANG=\"AMERICAN_AMERICA.\$ISO_SETTING\"\n";
        print SCRPT "fi\n";
        print SCRPT "\n";

        print SCRPT "if   [ \"\$NLS_LANG\" = \"\" ]; then\n";
        print SCRPT "   NLS_LANG=\$DEFAULT_NLS_LANG\n";
        print SCRPT "   export NLS_LANG\n";
        print SCRPT "else\n";
        print SCRPT "   NLS_LANG_STRING=`echo \$NLS_LANG | awk -F\".\" '{print \$1}'`\n";
        print SCRPT "   NLS_LANG_ISO=`echo \$NLS_LANG | awk -F\".\" '{print \$2}'`\n";
        print SCRPT "   if   [ \"\$NLS_LANG_ISO\" = \"\" ]; then\n";
        print SCRPT "           NLS_LANG=\$DEFAULT_NLS_LANG\n";
        print SCRPT "           export NLS_LANG\n";
        print SCRPT "   elif [ \"\$NLS_LANG_ISO\" != \$ISO_SETTING ]; then\n";
        print SCRPT "           NLS_LANG=\"\$NLS_LANG_STRING.\$ISO_SETTING\"\n";
        print SCRPT "           export NLS_LANG\n";
        print SCRPT "   fi\n";
        print SCRPT "fi\n";
	print SCRPT "# locale\n";
        print SCRPT "# echo \"Current NLS_LANG setting in .agprofile = \$NLS_LANG\"\n";
    }
    elsif ( $dbtype =~ /Microsoft/i ){
	print SCRPT
	"AGCLASSPATH=\$AGCLASSPATH:$msdir/msbase.jar:$msdir/mssqlserver.jar:$msdir/msutil.jar\n";
    }

    if ( $os_type =~ /AIX/i ){
	print SCRPT
	"AGCLASSPATH=\$AGCLASSPATH:\$SILVERSTREAMROOT/jre/lib/ext/classes\n";
    }

    close(SCRPT);

    chmod 0755, $filename;
}

###########################################################################
sub WriteScriptFiles{

    my $installdir = shift;
    my $dbname = shift;
    my $username = shift;
    my $password = shift;
    my $dbtype = shift;
    my $jversion = shift;
    my $classnamepath;

    my $classpath =
	$installdir . "/lib:".
	$installdir . "/lib/SilverServerAll.jar:".
	$installdir . "/lib/servlet.jar:".
	$installdir . "/lib/activation.jar:".
	$installdir . "/lib/mail.jar:".
	$installdir . "/lib/crimson.jar:".
	$installdir . "/lib/Cloudscape/cloudscape.jar:".
	$installdir . "/lib/Cloudscape/RmiJdbc.jar:".
	$installdir . "/lib/jaas.jar:".
	$installdir . "/lib/../jbrokerMQ/lib/jbroker-mq.jar:".
	$installdir . "/lib/ejb.jar:".
	$installdir . "/lib/ejb-portable.jar:".
	$installdir . "/lib/jdbc2_0-stdext.jar:".
	$installdir . "/lib/javax_trans.zip:".
	$installdir . "/lib/jms.jar:".
	$installdir . "/lib/Phaos_Security_Engine.jar:".
	$installdir . "/lib/Phaos_SSLava.jar:".
	$ENV{CLASSPATH} . ":\$AGCLASSPATH";

	#$installdir . "/lib/SilverDesignerAll.jar:".
	#$installdir . "/lib/SilverClasses.jar:".
	#$installdir . "/lib/ldap.jar:".
	#$installdir . "/lib/jndi.jar:".
	#$installdir . "/lib/nisplus.jar:".
	#$installdir . "/fts/lib/ssjdbc10.jar:".

	#$installdir . "/lib/rmi.zip:".
	#$installdir . "/lib/providerutil.jar:".
	#$installdir . "/lib/javax_ejb.zip:".
	#$installdir . "/lib/javax_jts.zip:".

    my $scriptfile;

    chdir $installdir . "/bin" or do {
	warn " ** Error: could not chdir $installdir/bin: $!\n";
	return 0;
    };

    my $java = "java";
    my $propsfile = "$installdir/$PROPS_PATH/$HPROPS_FILENAME";
    my $platformdbl = "$installdir/Resources/platforms.dbl";

    my $scriptheader = "#!$shell\n".
    "#\n".
    "# this script is generated by the SilverStream installation\n".
    "#\n".
    " . $installdir/$agprofile\n".
    "#set -x\n".
    "ulimit -n 1024\n\n";

    #---------------------------------------------------------------------------

    $scriptfile = $SCRIPTFILE_RENAMEDB;
    $classnamepath = $CLASSNAME_RENAMEDB;

    print " + Writing script file: $scriptfile ...\n";
    open(SCRIPTFILE, ">$scriptfile") or do {
	warn " ** Error: could not open: $scriptfile :$!\n";
	return 0;
	};

	#qq'dbhost="$DbHost"\n'.
    print SCRIPTFILE $scriptheader . "\n".
	qq'dbname="\$1" ; shift\n'.
	qq'username="\$1" ; shift\n'.
	qq'password="\$1" ; shift\n'.
	qq'newdbname="\$1" ; shift\n'.
	qq'\n'.
	qq'$java -classpath "$classpath"	\\\n'.
	qq'	$classnamepath			\\\n'.
	qq'	-v				\\\n'.
	qq'	-p "$propsfile"			\\\n'.
	qq'	-d "\$dbname"			\\\n'.
	qq'	-U "\$username" -P "\$password"	\\\n'.
	qq'	-n "\$newdbname"		\\\n'.
	qq'	"\$@"\n';
    close(SCRIPTFILE);

    chmod 0755, $scriptfile;

    return;
    # remove all this soon.
    #---------------------------------------------------------------------------
    # this last script is only for a bug in the HP java 116.
    # java 116 on HP can not use the executables, other version such as
    # java 115 and 117 can use the executables.
    if ( $os_type !~ /HP/i ){
	return;
    }

    my $script116;
    my $script115;
    my $javaopts = "-ss256K -ms16M -mx256M";

    #----------------------------------------------------------------------
    $scriptfile = $SCRIPTFILE_SSERVER;
    $classnamepath = $CLASSNAME_SSERVER;
    $script115 = "$scriptfile.1.1.5";
    $script116 = "$scriptfile.1.1.6";

    print " + Writing script file: $scriptfile ...\n";
    rename($scriptfile, "$script115");

    open(SCRIPTFILE, ">$scriptfile") or do {
	warn " ** Error: could not open: $scriptfile :$!\n";
	return 0;
	};

    print SCRIPTFILE $scriptheader . "\n".
	qq'export PATH="\$PATH:$installdir/fts"\n'.
	qq'$java $javaopts	\\\n'.
	qq'-classpath "$classpath"	\\\n'.
	qq'	$classnamepath		\\\n'.
	qq'	-p "$propsfile"		\\\n'.
	qq'	-SS_ROOT=$installdir	\\\n'.
	qq'	-SS_NOCONSOLE		\\\n'.
	qq'	"\$@"\n';
    close(SCRIPTFILE);

    chmod 0755, $scriptfile;
    rename($scriptfile, "$script116");

    if ( $jversion =~ m@1.1.6@){
	symlink($script116, $scriptfile);
    }
    else{
	symlink($script115, $scriptfile);
    }

    #----------------------------------------------------------------------
    $scriptfile = $SCRIPTFILE_SMINIT;
    $classnamepath = $CLASSNAME_SMINIT;
    $script115 = "$scriptfile.1.1.5";
    $script116 = "$scriptfile.1.1.6";

    print " + Writing script file: $scriptfile ...\n";
    rename($scriptfile, "$script115");

    open(SCRIPTFILE, ">$scriptfile") or do {
	warn " ** Error: could not open: $scriptfile :$!\n";
	return 0;
	};

    print SCRIPTFILE $scriptheader . "\n".
	qq'export PATH="\$PATH:$installdir/fts"\n'.
	qq'$java $javaopts	\\\n'.
	qq'-classpath "$classpath"	\\\n'.
	qq'	$classnamepath		\\\n'.
	qq'	-p "$propsfile"		\\\n'.
	qq'	-SS_ROOT=$installdir	\\\n'.
	qq'	-SS_NOCONSOLE		\\\n'.
	qq'	"\$@"\n';
    close(SCRIPTFILE);

    chmod 0755, $scriptfile;
    rename($scriptfile, "$script116");

    if ( $jversion =~ m@1.1.6@){
	symlink($script116, $scriptfile);
    }
    else{
	symlink($script115, $scriptfile);
    }
}

###########################################################################
sub RunSilverMasterInit{

    my $installdir = shift;
    my $username = shift;
    my $password = shift;
    my $loopsminit = 1;
    my $go = 1;
    my $sw;
    my $arg;
    my $masterName_save;

    my $cmd = "./SilverMasterInit";
    if ( $doUpgrade eq "yes"){
	$sw = "-r";
    }
    elsif ($restrictserver eq "yes"){
	$sw = "-f -A $adminuser -W $adminpasswd";
    }
    else{
	$sw = "-f -n -A $adminuser -W $adminpasswd";
    }

    while ($loopsminit){

	$loopsminit = 0;
	$arg = " $sw -U $username -P $password";

	print "\n". "-" x 70 . "\n";
	chdir("$installdir/bin") or
	    gracefulexit(10, " ** Error: Can not chdir to $installdir/bin: $!\n");

	print "   (current working dir: $installdir/bin)\n";
	print " + running: $cmd $arg\n";
	print "   **********************************************************\n".
	      "    This may take several minutes to complete depending upon\n".
	      "    the speed of your network connection, database server, \n".
	      "    and machine where you are running this install.\n".
	      "   **********************************************************\n";
	open(RUNIN, "$cmd $arg|") or
			gracefulexit(72, " ** Error: could not run: $cmd $arg:$!\n");
	while (<RUNIN>){
	    print;
	}
	close(RUNIN);

	if ( $? ) {
	    gracefulexit(72, " ** Error: $cmd failed. returned status: $?\n".
            "    A parameter previously entered may be incorrect.\n");
#RW        print "press <return> to continue, e=edit parameters, q=quit, r=retry... :";
#RW	    chomp(my $answ = <STDIN>);
#RW	    $answ =~ /^q/i && die "Exiting...\n";
#RW	    $loopsminit = 1 if $answ =~ /^r/i;
#RW	    if ( $answ =~ /^e/i ){
#RW		$masterName_save = $masterName;
#RW	      LABEL_LOOP_IN_SMASTERNAME:
#RW		if ( $go eq $BACK ){
#RW		    print "\n ** Can not go back from here **\n";
#RW		}
#RW		$mesg = "Please enter the name of your SilverMaster database.";
#RW		$go = &GetAnswer($mesg, \$masterName);
#RW		goto LABEL_LOOP_IN_SMASTERNAME if $go eq $BACK;

#RW	      LABEL_LOOP_IN_SMASTERUP:
#RW		($go, $username, $password) = &AskUserAndPass(1);
#RW		goto LABEL_LOOP_IN_SMASTERNAME if $go eq $BACK;
#RW		$loopsminit = 1;
#RW	    }
#RW	    print "\n";
#RW	    if ($masterName ne $masterName_save) {
#RW		&EditHttpdPropFile($installdir, $masterName);
#RW	    }
	}
    }
    return $?;
}

###########################################################################
sub RunAgUploadLicense{

    my $cmd = shift;
    my $srcdir = shift;
    my $installdir = shift;
    my $licstr = shift;
    my $silvermaster = shift;
    my $username = shift;
    my $password = shift;
    #my $shortlicensestring;

    if ( $os_type =~ /AIX/i ){
	system(qq'$cp "$srcdir/$licensesh" "$installdir/bin/$licensesh"');
	system(qq'$cp "$srcdir/$licensecmd" "$installdir/bin/$licensecmd"');
	$cmd = "$installdir/bin/$licensesh";
	chdir("$installdir/bin") or
	    gracefulexit(10, " ** Error: Can not chdir to $srcdir: $!\n");
    }
    else{
	$cmd = "$srcdir/$cmd";
	chdir($srcdir) or gracefulexit(10, " ** Error: Can not chdir to $srcdir: $!\n");
    }

    #$shortlicensestring = $licenseStr;
    #$shortlicensestring =~ s/\s+//g; # strip off white space
    #$shortlicensestring =~ s/........$//; # strip off the last 8 chars

    #my $arg = "'$shortlicensestring' +sscp:'$installdir' -C -U '$username' -P '$password' -d '$silvermaster'";
    my $arg = " $installdir ".
		"'$licstr' ".
		"-U '$username' ".
		"-P '$password'";

    print "\n". "-" x 70 . "\n";
    #chdir($installdir) or gracefulexit(10, " ** Error: Can not chdir to $installdir: $!\n");
    #chdir("bin");

    print "\n + running: $cmd $arg\n";
    open(RUNIN, "$shell $cmd $arg|") or gracefulexit(72, " ** Error: could not run: $cmd $arg:$!\n");
    while (<RUNIN>){
	print;
    }
    close(RUNIN);

    if ( $? ) {
	gracefulexit(72, " ** Error: $cmd failed.\n\treturned status: $?");
    }
    return $?;
}

###########################################################################
# execute the jbrokerMQ installer
sub InstalljbrokerMQ{

    my $srcd = shift;
    my $instd = shift;

    my $jbjar = "$instd/jbrokerMQ/lib/jbroker-mq.jar";

    # save in case the jbrokermq install fails
    print " + rename $jbjar ${jbjar}.save\n";
    rename($jbjar, "${jbjar}.save") or
	warn " ** Warning: could not rename: $!\n";

    my $java = "$instd/jre/bin/java";
    my $args = qq' -jar "$srcd/jbrokerMQ/jbrokerMQ.jar"'.
	    " -W target.dir=$instd/jbrokerMQ ".
	    " -W orb.install=false";

    if ($configjbmq eq "yes") {
	print " (DISPLAY=$ENV{DISPLAY})\n";
        print "Starting jBroker MQ configuration setup ...\n";
	$cmd = $java .
	" -Dcloudscape.system.home=$instd/jbrokerMQ/bin/$JB_PLATFORM_DIR".
	$args .
	" -W destination.active=false".
	" -W legal.display=false".
	" -W license.key=".
	'"jbroker-mq 2.0 0.0.0.0 forever 0 d189cbef-d03cbc49-fc2f9610"'.
	" -W orb.install=false -W db.migrate=true".
	" -W db.create=true";
    }
    else{
        print "Running jBroker MQ setup silently ... please wait ...\n";
	$cmd = $java . $args . " -silent";
    }

    print " + running: $cmd\n";
    system($cmd);

    # restore if the jbrokermq installer failed
    if ( -f $jbjar  && -f "${jbjar}.save"){
	print " + unlink ${jbjar}.save\n";
	unlink("${jbjar}.save");
    }
    elsif ( -f "${jbjar}.save" ){
	print " + rename ${jbjar}.save $jbjar\n";
	rename("${jbjar}.save", $jbjar);
    }
}

###########################################################################
sub RunAgDbRegistry{

    my $installdir = shift;
    my $username = shift;
    my $password = shift;
    my $smastername = shift;

    $smastername = "SilverMaster" unless $smastername;

    my $silver_reg_file = "$installdir/$registryfile";

    #----------------------------------------------------------------------
    # ppr 10230 do not run AgDbRegistry if SilverStream.reg already exists
    if ( -f $silver_reg_file){
	return;
    }

    my $command = $EXE_DBREGY;
    my $args =
	    qq' -add $smastername '.
	    qq' "$username" "$password" ';

    print "\n + Running: $command $args \n";
    open(USCRIPT, "./$command $args 2>&1|") or do {
	warn " ** Error: could not open $EXE_DBREGY: $!\n";
    };
    while(<USCRIPT>){
	print;
    }
    close(USCRIPT);
    print "\n";

}
###########################################################################
#
# 1st arg: the message displayed
# second arg is the variable name to be filled
#
sub GetAnswer {

    my $mesg = shift;
    my $var = shift;
    my $def = $$var;
    my @keys = @_;
    my $key = $keys[0];
    my $answ;

    LOOP_ASKQUESTION:
    {
	print "\n$mesg \n";
	print "(q=quit, b=back, default='$def') : ";

	chomp($answ = <STDIN>);
	$answ =~ /^q$/i && die "Exiting...\n";
	$answ =~ /^b$/i && return $BACK;
	$answ = $def unless $answ;

	if ( ! $answ && @keys == 1 && $key == $BLANK ){
	    last LOOP_ASKQUESTION;
	}
	elsif ( ! $answ ){
	    &Perr(" ** Invalid: please supply an answer.");
	}
	elsif ( @keys > 1 ){
	    for $k (@keys){
		if ( $answ eq $k ){
		    last LOOP_ASKQUESTION;
		}
	    }
	    &Perr(" ** Error: invalid option: $answ");
	}
	elsif ( ($key == $DIRECTORY || $key == $FILE) && $answ !~ m@^/@ ){
	    &Perr(" ** Error: directory is not absolute: $answ");
	}
	elsif ( $key == $DIRECTORY && ! -d $answ ){
	    &Perr(" ** Error: directory not found: $answ");
	}
	elsif ( $key == $FILE && ! -f $answ ){
	    &Perr(" ** Error: file not found: $answ");
	}
	elsif ( $key == $INTEGER && $answ !~ m@^\d+$@i ){
	    &Perr(" ** Invalid entry '$answ'. Please enter an integer.");
	}
	else {
	    last LOOP_ASKQUESTION;
	}
	redo LOOP_ASKQUESTION;

    } #LOOP_ASKQUESTION

    if ( $key == $YESNO ){
	if ( $answ =~ /y/i){
	    $answ = "yes";
	}
	else{
	    $answ = "no";
	}
    }

    $$var = $answ if $var;
    return $answ;
}

###########################################################################

sub Perr{
  print "\n".
	"    ************************************************************\n".
	"     @_\n".
	"    ************************************************************\n";
}

sub gracefulexit
{
    my ($errcode, $msg) = @_;
    Perr $msg;
    exit $errcode;
}
